@@ -1,6 +1,6 @@
 ;;; sb-asahi.el --- shimbun backend for asahi.com -*- coding: iso-2022-7bit; -*-
 
-;; Copyright (C) 2001-2011, 2013-2016 Yuuichi Teranishi <teranisi@gohome.org>
+;; Copyright (C) 2001-2011, 2013-2018 Yuuichi Teranishi <teranisi@gohome.org>
 
 ;; Author: TSUCHIYA Masatoshi <tsuchiya@namazu.org>,
 ;;         Yuuichi Teranishi  <teranisi@gohome.org>,
@@ -141,35 +141,30 @@ Every `.' in NAME will be replaced with `/'."
 	       (concat
 		"<item" s1 "rdf:about=\""
 		;; 1. url
-		"\\(http://\\(?:book\\|www\\)\\.asahi\\.com//?"
-		;; 2. extra keyword (en)
-		"\\([^/]+\\)"
-		"\\(?:/update/"
-		;; 3 and 4. serial number
-		"\\([0-9]+\\)\\)?/\\([0-9a-z]+\\)"
+		"\\(http://www.asahi\\.com/articles/"
+		;; 2. serial number
+		"\\([0-9a-z]+\\)"
 		"\\.html\\?ref=rss\\)"
 		"\"" s0 ">" s0 "<title>" s0
-		;; 5. subject
+		;; 3. subject
 		"\\([^<]+\\)"
-		s0 "</title>\\(?:"
-		s0 "\\(?:<[^>]+/>"
-		"\\|<[^>]+>\\(?:<[^]]+\\]+>\\|[^<]+\\)</[^>]+>\\)\\)*"
+		s0 "</title>\\(?:" s0 "<[^>]+>[^<>]*<" s0 "/[^>]+>\\)+"
 		s0 "<dc:subject>" s0
-		;; 6. extra keyword (ja)
+		;; 4. extra keyword (ja)
 		"\\([^<]+\\)"
 		s0 "</dc:subject>" s0 "<dc:date>" s0
-		;; 7. year
+		;; 5. year
 		"\\(20[0-9][0-9]\\)"
 		"-"
-		;; 8. month
+		;; 6. month
 		"\\([01][0-9]\\)"
 		"-"
-		;; 9. day
+		;; 7. day
 		"\\([0-3][0-9]\\)"
 		"T"
-		;; 10. hour:min:sec
+		;; 8. hour:min:sec
 		"\\([012][0-9]:[0-5][0-9]:[0-5][0-9]\\)")
-	       1 3 4 5 7 8 9 10 2 nil 6)))
+	       1 nil 2 3 5 6 7 8 nil nil 4)))
     `(("book" "RSS" "http://www3.asahi.com/rss/book.rdf" ,@rss)
       ("book.column" "$B%3%i%`(B")
       ("book.news" "$B=PHG%K%e!<%9(B" nil ,@book)
@@ -219,16 +214,18 @@ Every `.' in NAME will be replaced with `/'."
        1 nil 2 6 3 4 5)
       ("digital" "$B%G%8%?%k(B" "%s/list.html"
        ,@(shimbun-asahi-make-regexp "digital/[^\"/]+"))
-      ("editorial" "$B<R@b(B" "paper/editorial.html"
-       ,(concat "<meta" s1 "\\(?:[^\t\n >]+" s1 "\\)*name=\"RELEASE_DATE\""
-		s1 "\\(?:[^\t\n >]+" s1 "\\)*CONTENT=\""
-		;; 1. year
-		"\\(20[1-9][0-9]\\)-"
-		;; 2. month
-		"\\([01]?[0-9]\\)-"
-		;; 3. day
-		"\\([0-3]?[0-9]\\)\"")
-       nil nil nil nil 1 2 3)
+      ("editorial" "$B<R@b(B" "news/editorial.html"
+       ,(concat "<dd" s1 "data-num=\"[0-9]+\">" s0 "<a" s1 "href=\""
+		;; 1. url
+		"\\(/articles/"
+		;; 2. serial
+		"\\([^./]+\\)" "\\.html[^\"]*\\)" "[^>]*>" s0
+		"\\(?:<em>" s0
+		;; 3. subject
+		"\\([^<]+\\)" s0 "</em>.+\\|"
+		;; 4. subject
+		"\\([^<]+\\)" s0 "\\)</a>")
+       1 2 nil 3 nil nil nil nil nil nil nil nil nil 4)
       ("edu" "$B650i(B" "%s/list.html" ,@edu)
       ("english" "ENGLISH" "%s/index.html"
        ,@(let ((rest (shimbun-asahi-make-regexp "english.Herald-asahi")))
@@ -275,7 +272,7 @@ Every `.' in NAME will be replaced with `/'."
       ("sports.rugby" "$B%i%0%S!<(B")
       ("sports.usa" "$BJF%W%m%9%]!<%D(B")
       ("sports.winter" "$B%&%$%s%?!<%9%]!<%D(B")
-      ("tenjin" "$BE7@<?M8l(B" "paper/column.html"
+      ("tenjin" "$BE7@<?M8l(B" "paper/column.html" ;; WON'T WORK
        ,(concat "<meta" s1 "\\(?:[^\t\n >]+" s1 "\\)*name=\"RELEASE_DATE\""
 		s1 "\\(?:[^\t\n >]+" s1 "\\)*CONTENT=\""
 		;; 1. year
@@ -312,8 +309,9 @@ regexps and numbers.  Where index pages and regexps may contain the
 \"%s\" token which is replaced with group names, numbers point to the
 search result in order of [0]a url, [1,2]a serial number, [3]a subject,
 \[4]a year, [5]a month, [6]a day, [7]an hour:minute, [8,9,10]an extra
-keyword, [11]hour and [12]minute.  If an index page is nil, a group
-name in which \".\" is substituted with \"/\" is used instead.")
+keyword, [11]hour, [12]minute, and [13]an alternative subject.
+If an index page is nil, a group name in which \".\" is substituted with
+\"/\" is used instead.")
 
 (defvar shimbun-asahi-subgroups-alist
   (let* ((s0 "[\t\n $B!!(B]*")
@@ -925,7 +923,7 @@ Each table is the same as the `cdr' of the element of
 `shimbun-asahi-group-table'.")
 
 (defvar shimbun-asahi-content-start
-  "<!-+[\t\n ]*ArticleBody[\t\n ]*BGN[\t\n ]*-+>")
+  "<!-+[\t\n ]*ArticleTitle[\t\n ]*BGN[\t\n ]*-+>")
 
 (defvar shimbun-asahi-content-end
   "<!-+[\t\n ]*ArticleBody[\t\n ]*END[\t\n ]*-+>")
@@ -983,41 +981,11 @@ Face: iVBORw0KGgoAAAANSUhEUgAAAEIAAAAQBAMAAABQPLQnAAAAElBMVEX8rKjd3Nj+7utdXFr
 	   (shimbun-expand-url (format index group) shimbun-asahi-url)))))
 
 (defun shimbun-asahi-article-contents (group shimbun)
-  "Get article's contents for editorial and tenjin groups.
+  "Get article's contents for the tenjin group [WON'T WORK].
 Contents will be saved in the shimbun header as the extra element."
   (let ((case-fold-search t)
 	contents)
-    (cond ((string-equal group "editorial")
-	   (goto-char (point-max))
-	   (shimbun-retrieve-url "http://www.asahi.com/paper/editorial2.html")
-	   (let (tem)
-	     (goto-char (point-min))
-	     (while (and (re-search-forward "\
-<div[\t\n ]+\\(?:[^\t\n >]+[\t\n ]+\\)*class=\"ArticleTitle\"" nil t)
-			 (re-search-forward "\
-<div[\t\n ]+\\(?:[^\t\n >]+[\t\n ]+\\)*class=\"Title\"" nil t)
-			 (shimbun-end-of-tag "div")
-			 (progn
-			   (goto-char (match-beginning 2))
-			   (if (re-search-forward "<h[0-9]>[\t\n ]*\\([^<]+\\)"
-						  (match-end 2) t)
-			       (progn
-				 (push (shimbun-replace-in-string
-					(match-string 1) "[\t ]+\\'" "")
-				       tem)
-				 (re-search-forward "\
-<div[\t\n ]+\\(?:[^\t\n >]+[\t\n ]+\\)*class=\"ArticleText\"" nil t))
-			     (goto-char (match-end 0))
-			     nil))
-			 (shimbun-end-of-tag "div" t))
-	       (push (match-string 3) tem))
-	     (goto-char (point-max))
-	     (setq tem (nreverse tem))
-	     (while tem
-	       (setq contents (concat contents
-				      "<p><h2>" (pop tem) "</h2></p>\n"
-				      (pop tem) "\n")))))
-	  ((string-equal group "tenjin")
+    (cond ((string-equal group "tenjin")
 	   (goto-char (point-min))
 	   (when (and (re-search-forward "\
 <div[\t\n ]+\\(?:[^\t\n >]+[\t\n ]+\\)*class=\"ArticleBody\"" nil t)
@@ -1063,7 +1031,7 @@ Contents will be saved in the shimbun header as the extra element."
 	(from (concat (shimbun-server-name shimbun)
 		      " (" (shimbun-current-group-name shimbun) ")"))
 	(case-fold-search t)
-	regexp jname numbers book-p cyear cmonth rss-p paper-p en-category
+	regexp jname numbers book-p cyear cmonth cday rss-p paper-p en-category
 	hour-min month year day serial num extra rgroup id headers
 	travel-p subgroups iraq-p)
     (setq regexp (assoc group shimbun-asahi-group-table)
@@ -1077,6 +1045,7 @@ Contents will be saved in the shimbun header as the extra element."
 						    (substring group 5)
 						  group))))))
     (setq cyear (shimbun-decode-time nil 32400)
+	  cday (nth 3 cyear)
 	  cmonth (nth 4 cyear)
 	  cyear (nth 5 cyear)
 	  rss-p (member group '("book" "rss"))
@@ -1090,7 +1059,20 @@ Contents will be saved in the shimbun header as the extra element."
       (while t
 	(when regexp
 	  (while (re-search-forward regexp nil t)
-	    (cond ((string-equal group "english")
+	    (cond ((string-equal group "editorial")
+		   (let ((pt (point)))
+		     (save-match-data
+		       (if (re-search-backward "<li\\(?:[\t\n ]+[^\t\n >]+\\)*\
+[\t\n ]+data-date=\"[^\"0-9]*\\([01]?[0-9]\\)[^\"0-9]+\\([0-3]?[0-9]\\)" nil t)
+			   (progn
+			     (goto-char pt)
+			     (setq day (string-to-number (match-string 2))
+				   month (string-to-number (match-string 1))
+				   year (if (> month cmonth)
+					    (1- cyear)
+					  cyear)))
+			 (setq day cday month cmonth year cyear)))))
+		  ((string-equal group "english")
 		   (setq en-category
 			 (save-excursion
 			   (save-match-data
@@ -1108,24 +1090,26 @@ Contents will be saved in the shimbun header as the extra element."
 \\([012]?[0-9]:[0-5][0-9]\\))[\t\n ]*</span>"
 						    nil t)
 				 (match-string 1)))))))
-	    (setq month (string-to-number (match-string (nth 5 numbers)))
-		  year (if (setq num (nth 4 numbers))
-			   (string-to-number (match-string num))
-			 (cond ((>= (- month cmonth) 2)
-				(1- cyear))
-			       ((and (= 1 month) (= 12 cmonth))
-				(1+ cyear))
-			       (t
-				cyear)))
-		  day (string-to-number (match-string (nth 6 numbers)))
-		  serial (cond (rss-p
-				(if (match-beginning (nth 1 numbers))
+	    (unless (string-equal group "editorial")
+	      (setq month (string-to-number (match-string (nth 5 numbers)))
+		    year (if (setq num (nth 4 numbers))
+			     (string-to-number (match-string num))
+			   (cond ((>= (- month cmonth) 2)
+				  (1- cyear))
+				 ((and (= 1 month) (= 12 cmonth))
+				  (1+ cyear))
+				 (t
+				  cyear)))
+		    day (string-to-number (match-string (nth 6 numbers)))))
+	    (setq serial (cond (rss-p
+				(if (and (setq num (nth 1 numbers))
+					 (match-beginning num))
 				    (format "%d%s.%s"
 					    year
 					    (match-string (nth 1 numbers))
 					    (match-string (nth 2 numbers)))
 				  (match-string (nth 2 numbers))))
-			       (paper-p
+			       ((string-equal group "tenjin") ;; WON'T WORK
 				(format "%d%02d%02d" year month day))
 			       ((and (setq num (nth 1 numbers))
 				     (match-beginning num))
@@ -1153,7 +1137,7 @@ Contents will be saved in the shimbun header as the extra element."
 				 shimbun-asahi-top-level-domain ">")
 		       (concat "<" serial "%" rgroup "."
 			       shimbun-asahi-top-level-domain ">")))
-	    (setq extra (and paper-p
+	    (setq extra (and (string-equal group "tenjin") ;; WON'T WORK
 			     (shimbun-asahi-article-contents group shimbun)))
 	    (unless (shimbun-search-id shimbun id)
 	      (push (shimbun-create-header
@@ -1173,7 +1157,11 @@ Contents will be saved in the shimbun header as the extra element."
 				 (match-beginning num))
 			    (concat "[" (match-string num) "] "
 				    (match-string (nth 3 numbers))))
-			   (paper-p
+			   ((string-equal group "editorial")
+			    (format "%d/%d %s" month day
+				    (or (match-string (nth 3 numbers))
+					(match-string (nth 13 numbers)))))
+			   (paper-p ;; tenjin WON'T WORK
 			    (concat jname (format " (%d/%d)" month day)))
 			   (travel-p
 			    (save-match-data
@@ -1215,7 +1203,7 @@ Contents will be saved in the shimbun header as the extra element."
 		     ;; references, chars, lines
 		     "" 0 0
 		     ;; xref
-		     (if paper-p
+		     (if (string-equal group "tenjin") ;; WON'T WORK
 			 (shimbun-index-url shimbun)
 		       (shimbun-expand-url
 			(match-string (nth 0 numbers))
@@ -1279,7 +1267,7 @@ Contents will be saved in the shimbun header as the extra element."
 
 (defun shimbun-asahi-article (shimbun header outbuf)
   "Fetch article contents."
-  (if (member (shimbun-current-group-internal shimbun) '("editorial" "tenjin"))
+  (if (member (shimbun-current-group-internal shimbun) '("tenjin"));; WON'T WORK
       (let ((contents (cdr (assq 'Contents (shimbun-header-extra header)))))
 	(insert
 	 (with-temp-buffer
@@ -1312,8 +1300,8 @@ Contents will be saved in the shimbun header as the extra element."
 
 (defun shimbun-asahi-prepare-article (shimbun header)
   "Prepare an article.
-For the groups editorial and tenjin, use the extra element of HEADER
-as article contents."
+For the tenjin group [WON'T WORK], use the extra element of HEADER as
+article contents."
   (let ((case-fold-search t)
 	(group (shimbun-current-group-internal shimbun))
 	(from (shimbun-header-from-internal header)))
@@ -1346,7 +1334,9 @@ as article contents."
 		 (goto-char (match-beginning 0))
 		 (insert "\n<!-- End of Kiji -->\n"))))))
      ((member group '("editorial" "tenjin"))
-      )
+      (when (and (re-search-forward "<p class=\"SubTitle\"" nil t)
+		 (shimbun-end-of-tag "p" t))
+	(replace-match "\n")))
      ((string-equal group "food")
       (when (and (re-search-forward (shimbun-content-start shimbun) nil t)
 		 (re-search-forward "[\t\n ]*<!-+[\t\n ]+Creative[\t\n ]+for"
@@ -1481,11 +1471,20 @@ as article contents."
 
 (defun shimbun-asahi-clear-contents (shimbun header)
   (when (luna-call-next-method)
-    ;; Remove garbage after an article.
+    ;; Remove garbage between the title and the body.
     (goto-char (point-min))
-    (when (re-search-forward
-	   "[\t\n ]*<!-+[\t\n ]*main[\t\n ]+text[\t\n ]+end[\t\n ]*-+>"
-	   nil t)
+    (when (re-search-forward "\\(?:[\t\n ]*<[^>]+>\\)*\
+<!-+[\t\n ]*ArticleTitle[\t\n ]*END[\t\n ]*-+>" nil t)
+      (let ((st (match-beginning 0)))
+	(when (re-search-forward
+	       "<!-+[\t\n ]*ArticleBody[\t\n ]*BGN[\t\n ]*-+>" nil t)
+	  (delete-region st (match-end 0))
+	  (insert "\n")))
+      (goto-char (point-min)))
+    ;; Remove garbage after an article.
+    (when (re-search-forward "[\t\n ]*\
+<!-+[\t\n ]*\\(?:article\\|main\\)[\t\n ]*text[\t\n ]+end[\t\n ]*-+>"
+			     nil t)
       (delete-region (match-beginning 0) (point-max))
       (goto-char (point-min)))
     ;; Remove forms and links.
@@ -1542,6 +1541,11 @@ as article contents."
     (when (re-search-forward "\
 [\t\n ]*<!-+[\t\n ]*Outbrain[\t\n ]+TAG[\t\n ]+PC[\t\n ]*-+>" nil t)
       (delete-region (match-end 0) (point-max)))
+    (goto-char (point-min))
+    (while (re-search-forward "<div\\(?:[\t\n ]+[^\t\n >]+\\)*[\t\n ]+\
+class=\"ExtendedLinkMod\"" nil t)
+      (when (shimbun-end-of-tag "div" t)
+	(replace-match "\n")))
     ;; Remove Ads.
     (goto-char (point-min))
     (let (st)
@@ -1552,8 +1556,6 @@ as article contents."
 		    (re-search-forward
 		     "<!-+[\t\n ]*Ad[\t\n ]+END[\t\n ]*-+>[\t\n ]*" nil t)))
 	(delete-region st (match-end 0))))
-    (unless (shimbun-prefer-text-plain-internal shimbun)
-      (shimbun-break-long-japanese-lines))
     t))
 
 (luna-define-method shimbun-clear-contents :around ((shimbun shimbun-asahi)
