@@ -1,6 +1,6 @@
 ;;; sb-mainichi.el --- shimbun backend for Mainichi jp -*- coding: iso-2022-7bit; -*-
 
-;; Copyright (C) 2001-2009, 2011-2013, 2015, 2016
+;; Copyright (C) 2001-2009, 2011-2013, 2015-2018
 ;; Koichiro Ohba <koichiro@meadowy.org>
 
 ;; Author: Koichiro Ohba <koichiro@meadowy.org>
@@ -38,7 +38,7 @@
 (luna-define-class shimbun-mainichi (shimbun-japanese-newspaper
 				     shimbun-multi shimbun-rss) ())
 
-(defvar shimbun-mainichi-url "http://mainichi.jp/")
+(defvar shimbun-mainichi-url "https://mainichi.jp/")
 
 (defvar shimbun-mainichi-top-level-domain "mainichi.jp")
 
@@ -54,43 +54,33 @@
   (shimbun-rss-initialize-ignored-subject shimbun))
 
 (defvar shimbun-mainichi-group-table
-  '(("flash" "$B%K%e!<%9B.Js(B"
-     "http://mainichi.jp/rss/etc/flash.rss")
+  '(("flash" "$B%K%e!<%9B.Js(B($BAm9g(B)"
+     "https://mainichi.jp/rss/etc/mainichi-flash.rss")
     ("sports" "$B%9%]!<%D(B"
-     "http://mainichi.jp/rss/etc/sports.rss")
-    ("entertainment" "$B%(%s%?!<%F%$%s%a%s%H(B"
-     "http://mainichi.jp/rss/etc/enta.rss")
-    ("mantan" "$B%"%K%a!&%^%s%,!&%2!<%`(B"
-     "http://mainichi.jp/rss/etc/mantan.rss")
-    ("electronics" "$B#I#T!&2HEE(B"
-     "http://mainichi.jp/rss/etc/electronics.rss")
-    ("weekly" "$B1Q8l$r3X$V(B"
-     "http://mainichi.jp/rss/etc/weekly.rss")
+     "https://mainichi.jp/rss/etc/mainichi-sports.rss")
+    ("entertainment" "$B%(%s%?%a(B"
+     "https://mainichi.jp/rss/etc/mainichi-enta.rss")
+    ("opinion" "$B<R@b!&2r@b!&%3%i%`(B"
+     "https://mainichi.jp/rss/etc/opinion.rss")
     ;; Non-RSS groups.
     ("opinion.editorial" "$B<R@b(B"
-     "http://mainichi.jp/select/opinion/editorial/archive/")
+     "https://mainichi.jp/editorial/")
     ("opinion.yoroku" "$BM>O?(B"
-     "http://mainichi.jp/select/opinion/yoroku/archive/")
+     "https://mainichi.jp/yoroku/")
     ("opinion.hasshinbako" "$BH/?.H"(B"
-     "http://mainichi.jp/select/opinion/hasshinbako/archive/")
+     "https://mainichi.jp/hasshinbako/")
     ("opinion.eye" "$B5-<T$NL\(B"
-     "http://mainichi.jp/select/opinion/eye/archive/")
+     "https://mainichi.jp/kishanome/")
     ("opinion.hito" "$B$R$H(B"
-     "http://mainichi.jp/select/opinion/hito/archive/")
+     "https://mainichi.jp/hito/")
     ("opinion.kinji" "$B6a;vJR!9(B"
-     "http://mainichi.jp/select/opinion/kinji/archive/")
+     "https://mainichi.jp/kinji/")
     ("opinion.yuraku" "$BM+3ZD"(B"
-     "http://mainichi.jp/select/opinion/yuraku/archive/")
-    ("opinion.closeup" "$B%/%m!<%:%"%C%W(B"
-     "http://mainichi.jp/select/opinion/closeup/archive/")
-    ("opinion.kaisetsu" "$BEZMK2r@b(B"
-     "http://mainichi.jp/select/opinion/kaisetsu/")
-    ("opinion.newsup" "$B%K%e!<%9#U#P(B"
-     "http://mainichi.jp/select/opinion/newsup/")
+     "https://mainichi.jp/yurakucho/")
     ("opinion.jidainokaze" "$B;~Be$NIw(B"
-     "http://mainichi.jp/select/opinion/jidainokaze/")
+     "https://mainichi.jp/jidainokaze/")
     ("entertainment.art" "$B7]=Q!&J82=(B"
-     "http://mainichi.jp/enta/art/archive/")
+     "https://mainichi.jp/art/")
     ("fuchisou" "$BIwCNAp(B"
      "http://mainichi.jp/fuchisou/")))
 
@@ -107,6 +97,26 @@ Face: iVBORw0KGgoAAAANSUhEUgAAABwAAAAcBAMAAACAI8KnAAAABGdBTUEAALGPC/xhBQAAABh
 
 (defvar shimbun-mainichi-expiration-days 7)
 
+;; The login stuff no longer works; it now requires to run java-script.
+
+;;(defvar shimbun-mainichi-login-url "https://mainichi.jp/auth/login.php"
+;;  "*Url base to login to mainichi.jp.")
+
+;;(defvar shimbun-mainichi-logout-url "https://mainichi.jp/auth/logout.php"
+;;  "*Url base to logout from mainichi.jp.")
+
+;;(defcustom shimbun-mainichi-login-name nil
+;;  "Login name used to login to mainichi.jp.
+;;To use this, set both `w3m-use-cookies' and `w3m-use-form' to t."
+;;  :group 'shimbun
+;;  :type '(choice (const :tag "None" nil) (string :tag "User name")))
+
+;;(defcustom shimbun-mainichi-login-password nil
+;;  "Password used to login to mainichi.jp.
+;;To use this, set both `w3m-use-cookies' and `w3m-use-form' to t."
+;;  :group 'shimbun
+;;  :type '(choice (const :tag "None" nil) (string :tag "Password")))
+
 (luna-define-method shimbun-groups ((shimbun shimbun-mainichi))
   (mapcar 'car shimbun-mainichi-group-table))
 
@@ -183,7 +193,7 @@ Face: iVBORw0KGgoAAAANSUhEUgAAABwAAAAcBAMAAACAI8KnAAAABGdBTUEAALGPC/xhBQAAABh
 		    (concat
 		     "<a[\t\n ]+\\(?:[^\t\n >]+[\t\n ]+\\)*href=\""
 		     ;; 1. url
-		     "\\(/articles/"
+		     "\\(\\(?://?[^\"/>]+\\)*/articles/"
 		     ;; 2. serial number
 		     "\\("
 		     ;; 3. year
@@ -263,9 +273,9 @@ Face: iVBORw0KGgoAAAANSUhEUgAAABwAAAAcBAMAAACAI8KnAAAABGdBTUEAALGPC/xhBQAAABh
 	(shimbun-strip-cr)
 	(goto-char (point-min))
 	(when (and (re-search-forward "\
-<div[\t\n ]+\\(?:[^\t\n >]+[\t\n ]+\\)*class=\"NewsArticle\""
+<section[\t\n ]+\\(?:[^\t\n >]+[\t\n ]+\\)*class=\"newslist\""
 				      nil t)
-		   (shimbun-end-of-tag "div"))
+		   (shimbun-end-of-tag "section"))
 	  (goto-char (match-beginning 0))
 	  (setq end (match-end 0))
 	  (while (re-search-forward
@@ -282,12 +292,14 @@ Face: iVBORw0KGgoAAAANSUhEUgAAABwAAAAcBAMAAACAI8KnAAAABGdBTUEAALGPC/xhBQAAABh
 		     "\\([01][0-9]\\)"
 		     ;; 5. day
 		     "\\([0-3][0-9]\\)"
-		     "[^\".]+\\)\\.html\\)\""
+		     "[^\".]+\\)\\)\""
 		     "\\(?:[\t\n ]+[^\t\n >]+\\)*[\t\n ]*>[\t\n ]*"
+		     "\\(?:<[^>]+>[\t\n ]*\\)*"
 		     ;; 6. subject
 		     "\\([^<]+\\)"))
 		  end t)
-	    (setq id (concat "<" (match-string 2) "." rgrp
+	    (setq id (concat "<" (subst-char-in-string ?/ ?. (match-string 2))
+			     "." rgrp
 			     "%" shimbun-mainichi-top-level-domain ">"))
 	    (if (shimbun-search-id shimbun id)
 		(unless (zerop count)
@@ -376,7 +388,8 @@ Face: iVBORw0KGgoAAAANSUhEUgAAABwAAAAcBAMAAACAI8KnAAAABGdBTUEAALGPC/xhBQAAABh
   (if (and (re-search-forward
 	    "<div[\t\n ]+\\(?:[^\t\n >]+[\t\n ]+\\)*class=\"main-text\"" nil t)
 	   (shimbun-end-of-tag "div"))
-      (let ((hankaku (shimbun-japanese-hankaku shimbun))
+      (let ((group (shimbun-current-group-internal shimbun))
+	    (hankaku (shimbun-japanese-hankaku shimbun))
 	    regexp)
 	(delete-region (match-end 2) (point-max))
 	(delete-region (point-min) (match-beginning 2))
@@ -388,6 +401,10 @@ Face: iVBORw0KGgoAAAANSUhEUgAAABwAAAAcBAMAAACAI8KnAAAABGdBTUEAALGPC/xhBQAAABh
 	    (when (shimbun-end-of-tag "div" t)
 	      (delete-region (goto-char (match-beginning 0)) (match-end 0))
 	      (insert "\n"))))
+	(when (string-equal group "opinion.yoroku")
+	  (goto-char (point-min))
+	  (while (search-forward "$B"%(B" nil t)
+	    (replace-match "$B!#(B</p><p>$B!!(B")))
 	;; Convert Japanese zenkaku ASCII chars into hankaku.
 	(when (and hankaku (not (memq hankaku '(header subject))))
 	  (shimbun-japanese-hankaku-buffer t))
@@ -399,9 +416,7 @@ Face: iVBORw0KGgoAAAANSUhEUgAAABwAAAAcBAMAAACAI8KnAAAABGdBTUEAALGPC/xhBQAAABh
 \\(?:[^\t\n >]+[\t\n ]+\\)*alt=\"\\)[^\"]+"
 					     nil t)
 			  (shimbun-end-of-tag nil t))
-		(replace-match "\n&lt;$B<L??(B&gt;\n")))
-	  ;; Break long lines.
-	  (shimbun-break-long-japanese-lines))
+		(replace-match "\n&lt;$B<L??(B&gt;\n"))))
 	t)
 ;;    (erase-buffer)
 ;;    (insert "<html><body>\
@@ -427,6 +442,177 @@ Face: iVBORw0KGgoAAAANSUhEUgAAABwAAAAcBAMAAACAI8KnAAAABGdBTUEAALGPC/xhBQAAABh
       (insert "&#012;\n"))
     t))
 
+(eval-when-compile
+  (require 'w3m-cookie)
+  (require 'w3m-form))
+
+;;(autoload 'password-cache-add "password-cache")
+;;(autoload 'password-read-from-cache "password-cache")
+
+;;(defun shimbun-mainichi-login (&optional name password interactive-p)
+;;  "Login to mainichi.jp with NAME and PASSWORD.
+;;NAME and PASSWORD default to `shimbun-mainichi-login-name' and
+;;`shimbun-mainichi-login-password' respectively.  `password-data', if
+;;cached, overrides `shimbun-mainichi-login-password'.  If the prefix
+;;argument is given, you will be prompted for new NAME and PASSWORD."
+;;  (interactive (let ((pass (copy-sequence shimbun-mainichi-login-password))
+;;		     name default password)
+;;		 (unless (and w3m-use-cookies w3m-use-form)
+;;		   (error "\
+;;You should set `w3m-use-cookies' and `w3m-use-form' to non-nil"))
+;;		 (setq name (if current-prefix-arg
+;;				(completing-read
+;;				 "Login name: "
+;;				 (cons shimbun-mainichi-login-name nil)
+;;				 nil nil shimbun-mainichi-login-name)
+;;			      shimbun-mainichi-login-name))
+;;		 (when (and name (string-match "\\`[\t ]*\\'" name))
+;;		   (setq name nil))
+;;		 (setq default (and name
+;;				    (or (password-read-from-cache name)
+;;					;; `password-cache' will expire
+;;					;; the password by filling it with
+;;					;; C-@'s, so we use a copy of
+;;					;; the original.
+;;					(copy-sequence
+;;					 shimbun-mainichi-login-password)))
+;;		       password (and name
+;;				     (if current-prefix-arg
+;;					 (read-passwd
+;;					  (concat "Password"
+;;						  (when default
+;;						    (concat " (default "
+;;							    (make-string
+;;							     (length default)
+;;							     ?*)
+;;							    ")"))
+;;						  ": ")
+;;					  nil default)
+;;				       default)))
+;;		 (when (and password (string-match "\\`[\t ]*\\'" password))
+;;		   (setq name nil
+;;			 password nil))
+;;		 (list name password t)))
+;;  (unless interactive-p
+;;    (if (or name (setq name shimbun-mainichi-login-name))
+;;	(or password
+;;	    (setq password
+;;		  (or (password-read-from-cache name)
+;;		      (copy-sequence shimbun-mainichi-login-password)))
+;;	    (setq name nil))
+;;      (setq password nil)))
+;;  (if (not (and w3m-use-cookies w3m-use-form name password))
+;;      (when interactive-p (message "Quit"))
+;;    (require 'w3m-cookie)
+;;    (require 'w3m-form)
+;;    (let ((cache (buffer-live-p w3m-cache-buffer))
+;;	  (num 0)
+;;	  temp form plist val uid pass handler)
+;;      (condition-case err
+;;	  (w3m-process-do-with-temp-buffer
+;;	      (type (progn
+;;		      (setq temp (current-buffer))
+;;		      (w3m-retrieve shimbun-mainichi-url nil t)))
+;;	    (if (not type)
+;;		(when interactive-p (message "Failed to login"))
+;;	      (goto-char (point-min))
+;;	      (if (not (re-search-forward
+;;			"<a[\t\n ]+href=\"/auth/login\\.php\\?url="
+;;			nil t))
+;;		  (when interactive-p (message "Already logged in"))
+;;		(erase-buffer)
+;;		(set-buffer-multibyte t)
+;;		(w3m-process-with-wait-handler
+;;		  (w3m-retrieve-and-render
+;;		   (concat shimbun-mainichi-login-url "?url="
+;;			   (shimbun-url-encode-string shimbun-mainichi-url))
+;;		   t nil nil shimbun-mainichi-url handler))
+;;		(setq form (car w3m-current-forms))
+;;		(if (not (string-match "/auth\\.mainichi\\.co\\.jp/"
+;;				       (w3m-form-action form)))
+;;		    (when interactive-p
+;;		      (message "Already logged in"))
+;;		  (setq plist (w3m-form-plist form))
+;;		  (while (and (not (and uid pass)) (setq val (pop plist)))
+;;		    (if (numberp val)
+;;			(setq num (max num val))
+;;		      (when (eq (car val) :value)
+;;			(setq val (cadr val))
+;;			(cond ((string-equal "uidemail" (car val))
+;;			       (setcdr val name)
+;;			       (setq uid t))
+;;			      ((string-equal "password" (car val))
+;;			       (setcdr val password)
+;;			       (setq pass t))))))
+;;		  (unless uid
+;;		    (nconc (setq plist (w3m-form-plist form))
+;;			   (list (setq num (1+ num))
+;;				 (list :value (cons "uidemail" name)))))
+;;		  (unless pass
+;;		    (nconc (or plist (w3m-form-plist form))
+;;			   (list (setq num (1+ num))
+;;				 (list :value (cons "password" password)))))
+;;		  (erase-buffer)
+;;		  (w3m-process-with-wait-handler
+;;		    (w3m-retrieve-and-render
+;;		     (w3m-form-action form) t nil
+;;		     (w3m-form-make-form-data form)
+;;		     shimbun-mainichi-login-url handler))
+;;		  (if (not (string-equal w3m-current-url shimbun-mainichi-url))
+;;		      (when interactive-p (message "Failed to login"))
+;;		    (when interactive-p (message "Logged in"))
+;;		    (password-cache-add name password)
+;;		    (when w3m-cookie-save-cookies (w3m-cookie-save))))))
+;;	    (when (get-buffer " *w3m-cookie-parse-temp*")
+;;	      (kill-buffer (get-buffer " *w3m-cookie-parse-temp*")))
+;;	    (unless cache (w3m-cache-shutdown)))
+;;	(error (if (or interactive-p debug-on-error)
+;;		   (signal (car err) (cdr err))
+;;		 (message "Error while logging in to mainichi.jp:\n %s"
+;;			  (error-message-string err))
+;;		 (when (buffer-live-p temp) (kill-buffer temp))))))))
+
+;;(defun shimbun-mainichi-logout (&optional interactive-p)
+;;  "Logout from mainichi.jp."
+;;  (interactive (list t))
+;;  (require 'w3m-cookie)
+;;  (require 'w3m-form)
+;;  (let ((cache (buffer-live-p w3m-cache-buffer))
+;;	temp handler)
+;;    (condition-case err
+;;	(w3m-process-do-with-temp-buffer
+;;	    (type (progn
+;;		    (setq temp (current-buffer))
+;;		    (w3m-retrieve shimbun-mainichi-url nil t)))
+;;	  (if (not type)
+;;	      (when interactive-p (message "Failed to logout"))
+;;	    (goto-char (point-min))
+;;	    (if (not (re-search-forward
+;;		      "<a[\t\n ]+href=\"/auth/logout\\.php\\?url="
+;;		      nil t))
+;;		(when interactive-p (message "Already logged out"))
+;;	      (erase-buffer)
+;;	      (set-buffer-multibyte t)
+;;	      (w3m-process-with-wait-handler
+;;		(w3m-retrieve-and-render
+;;		 (concat shimbun-mainichi-logout-url "?url="
+;;			 (shimbun-url-encode-string shimbun-mainichi-url))
+;;		 t nil nil shimbun-mainichi-url handler))
+;;	      (if (not (string-equal w3m-current-url shimbun-mainichi-url))
+;;		  (when interactive-p (message "Failed to logout"))
+;;		(when interactive-p (message "Logged out"))
+;;		(when w3m-cookie-save-cookies (w3m-cookie-save)))))
+;;	  (when (get-buffer " *w3m-cookie-parse-temp*")
+;;	    (kill-buffer (get-buffer " *w3m-cookie-parse-temp*")))
+;;	  (unless cache (w3m-cache-shutdown)))
+;;      (error (if (or interactive-p debug-on-error)
+;;		 (signal (car err) (cdr err))
+;;	       (message "Error while logging out from mainichi.jp:\n %s"
+;;			(error-message-string err))
+;;	       (when (buffer-live-p temp) (kill-buffer temp)))))))
+
+;;(shimbun-mainichi-login)
+
 (provide 'sb-mainichi)
 
 ;;; sb-mainichi.el ends here
