@@ -1,6 +1,7 @@
 ;;; w3m-bookmark.el --- Functions to operate bookmark file of w3m
 
-;; Copyright (C) 2001-2003, 2005-2012 TSUCHIYA Masatoshi <tsuchiya@namazu.org>
+;; Copyright (C) 2001-2003, 2005-2012, 2017
+;; TSUCHIYA Masatoshi <tsuchiya@namazu.org>
 
 ;; Authors: Shun-ichi GOTO     <gotoh@taiyo.co.jp>,
 ;;          TSUCHIYA Masatoshi <tsuchiya@namazu.org>
@@ -43,7 +44,7 @@
   (expand-file-name "bookmark.html" w3m-profile-directory)
   "Bookmark file of w3m."
   :group 'w3m
-  :type '(file :size 0))
+  :type 'file)
 
 (defcustom w3m-bookmark-file-coding-system 'euc-japan
   "Coding system for a created bookmark file.
@@ -53,14 +54,14 @@ system which is used to encode your using bookmark file is different
 from the value of this option, emacs-w3m does not change the encoding
 of your bookmark file."
   :group 'w3m
-  :type '(coding-system :size 0))
+  :type 'coding-system)
 
 (defcustom w3m-bookmark-default-section
   nil
   "Default section to add new entry."
   :group 'w3m
   :type '(radio (const :tag "Not specified" nil)
-		(string :format "Default section name: %v\n" :size 0)))
+		(string :format "Default section name: %v\n")))
 
 (defcustom w3m-bookmark-mode-hook nil
   "*Hook run at the end of function `w3m-bookmark-mode'."
@@ -177,9 +178,7 @@ file exists, otherwise nil."
 	  (setq w3m-bookmark-buffer-file-name w3m-bookmark-file)
 	  (set-visited-file-modtime (or (w3m-bookmark-file-modtime)
 					;; No bookmark file.
-					(with-temp-buffer
-					  ;; 0 in Emacs; (0 . 0) in XEmacs
-					  (visited-file-modtime))))
+					'(0 0)))
 	  (buffer-enable-undo))
 	(current-buffer)))))
 
@@ -237,7 +236,8 @@ file exists, otherwise nil."
 	  (file-error nil)))))))
 
 (defun w3m-bookmark-safe-string (string format)
-  (labels ((filter (s c) (decode-coding-string (encode-coding-string s c) c)))
+  (w3m-labels ((filter (s c)
+		       (decode-coding-string (encode-coding-string s c) c)))
     (if (let ((encoding (w3m-static-when (featurep 'mule)
 			  buffer-file-coding-system)))
 	  (or (string= string (filter string encoding))
@@ -359,7 +359,7 @@ With prefix, ask for a new url instead of the present one."
 
 ;;;###autoload
 (defun w3m-bookmark-view (&optional reload)
-  "Display the bookmark."
+  "Display the bookmark list in the current buffer."
   (interactive "P")
   (if (file-exists-p w3m-bookmark-file)
       (progn
@@ -370,7 +370,7 @@ With prefix, ask for a new url instead of the present one."
 
 ;;;###autoload
 (defun w3m-bookmark-view-new-session (&optional reload)
-  "Display the bookmark on a new session."
+  "Display the bookmark list in a new buffer."
   (interactive "P")
   (if (not (eq major-mode 'w3m-mode))
       (message "This command can be used in w3m mode only")
