@@ -1,6 +1,6 @@
 ;;; w3m-form.el --- Stuffs to handle <form> tag
 
-;; Copyright (C) 2001-2012 TSUCHIYA Masatoshi <tsuchiya@namazu.org>
+;; Copyright (C) 2001-2014, 2017 TSUCHIYA Masatoshi <tsuchiya@namazu.org>
 
 ;; Authors: TSUCHIYA Masatoshi <tsuchiya@namazu.org>,
 ;;          Yuuichi Teranishi  <teranisi@gohome.org>,
@@ -71,7 +71,7 @@ Files to save text are stored in the directory specified by the
 		 (const :tag "No expire" nil)))
 
 (defcustom w3m-form-textarea-file-coding-system
-  (cond ((or (featurep 'un-define) 
+  (cond ((or (featurep 'un-define)
 	     (fboundp 'utf-translate-cjk-mode))
 	 'utf-8)
 	((equal "Japanese" w3m-language)
@@ -82,20 +82,21 @@ Files to save text are stored in the directory specified by the
 	 'iso-2022-7bit-ss2))
   "Coding system for textarea's backup file."
   :group 'w3m
-  :type '(coding-system :size 0))
+  :type 'coding-system)
 
 (defcustom w3m-form-textarea-directory
   (expand-file-name ".textarea" w3m-profile-directory)
   "*Name of the directory to save the file of textarea input."
   :group 'w3m
-  :type '(directory :size 0))
+  :type 'directory)
 
 (defcustom w3m-form-textarea-edit-mode 'text-mode
   "*Major mode to edit textarea."
   :group 'w3m
   :type '(choice
 	  (function :tag "Major mode")
-	  (repeat :tag "Rules to select major modes for the current page"
+	  (repeat
+	   :tag "Rules to select major modes for the current page"
 	   (cons (choice (regexp :tag "Regexp matches the current page")
 			 (function :tag "Predicate checks the current page")
 			 (sexp :tag "Expression checks the current page"))
@@ -110,6 +111,19 @@ Files to save text are stored in the directory specified by the
 ;; backward-compatibility alias
 (put 'w3m-form-face 'face-alias 'w3m-form)
 
+(defface w3m-form-inactive
+  ;; A copy of the `shadow' face, plus `underline'.
+  '((((class color grayscale) (min-colors 88) (background light))
+     (:foreground "grey50" :underline t))
+    (((class color grayscale) (min-colors 88) (background dark))
+     (:foreground "grey70" :underline t))
+    (((class color) (min-colors 8) (background light))
+     (:foreground "green" :underline t))
+    (((class color) (min-colors 8) (background dark))
+     (:foreground "yellow" :underline t)))
+  "Face used to dim inactive forms."
+  :group 'w3m-face)
+
 ;;; Local variables
 (defvar w3m-form-input-textarea-buffer nil)
 (defvar w3m-form-input-textarea-form nil)
@@ -234,15 +248,16 @@ It is useful to bind this variable with `let', but do not set it globally.")
   `(w3m-form-put-property ,form ,id :value (cons ,name ,value)))
 (defmacro w3m-form-get (form id)
   `(cdr (w3m-form-get-property ,form ,id :value)))
-(defun w3m-form-get-by-name (form name)
+(defun w3m-form-get-by-name (form id name)
   (let ((plist (w3m-form-plist form))
 	pair value)
     (while plist
-      (setq pair (plist-get (cadr plist) :value))
-      (when (and pair
-		 (string= (car pair) name))
-	(setq value (cdr pair)
-	      plist nil))
+      (when (eq id (car plist))
+	(setq pair (plist-get (cadr plist) :value))
+	(when (and pair
+		   (string= (car pair) name))
+	  (setq value (cdr pair)
+		plist nil)))
       (setq plist (cddr plist)))
     value))
 (defun w3m-form-put-by-name (form id name value)
@@ -286,6 +301,7 @@ If no field in forward, return nil without moving."
 (defun w3m-form-make-form-data (form)
   (let ((plist (w3m-form-plist form))
 	(coding (w3m-form-charlst form))
+	(w3m-cookie-p (equal "about://cookie/" (w3m-form-action form)))
 	buf bufs)
     (setq coding (w3m-form-get-coding-system coding))
     (while plist
@@ -308,7 +324,7 @@ If no field in forward, return nil without moving."
 	  (setq bufs (cons (cons number (cons name value)) bufs))))
 	(setq plist (cddr plist))))
     (when bufs
-      (setq bufs (sort bufs (lambda (x y) (< (car x) (car y)))))
+      (setq bufs (sort bufs #'car-less-than-car))
       (if (eq (w3m-form-enctype form) 'multipart/form-data)
 	  (let ((boundary (apply 'format "--_%d_%d_%d" (current-time)))
 		file type)
@@ -339,7 +355,9 @@ If no field in forward, return nil without moving."
 			   "Content-Disposition: form-data; name=\""
 			   (car buf)
 			   "\"\r\n\r\n"
-			   (encode-coding-string (cdr buf) coding)
+			   (if (cdr buf)
+			       (encode-coding-string (cdr buf) coding)
+			     "")
 			   "\r\n"))
 		 (setq bufs (cdr bufs)))
 	       (insert "--" boundary "--\r\n")
@@ -348,11 +366,13 @@ If no field in forward, return nil without moving."
 		     (setq elem (cdr elem))
 		     (format "%s=%s"
 			     (w3m-url-encode-string (car elem) coding t)
-			     (w3m-url-encode-string (if (stringp (cdr elem))
-							(cdr elem)
-						      "")
-						    coding
-						    t)))
+			     (w3m-url-encode-string
+			      (if (stringp (cdr elem))
+				  (cdr elem)
+				"")
+			      coding
+			      (not (and w3m-cookie-p
+					(equal (car elem) "search"))))))
 		   bufs "&")))))
 
 (defun w3m-form-resume (forms)
@@ -393,7 +413,7 @@ fid=\\([^/]+\\)/type=\\([^/]+\\)/name=\\([^/]*\\)/id=\\(.*\\)$"
 		    (unless (eq form cform)
 		      (w3m-form-put cform id name (w3m-form-get form id))))
 		   ((string= type "radio")
-		    (let ((value (w3m-form-get-by-name form name)))
+		    (let ((value (w3m-form-get-by-name form id name)))
 		      (when value
 			(w3m-form-replace
 			 (if (string= value (nth 4 (w3m-action (point))))
@@ -450,10 +470,13 @@ fid=\\([^/]+\\)/type=\\([^/]+\\)/name=\\([^/]*\\)/id=\\(.*\\)$"
 
 (eval-and-compile
   (unless (fboundp 'w3m-form-make-button)
-    (defun w3m-form-make-button (start end properties)
+    (defun w3m-form-make-button (start end properties &optional readonly)
       "Make button on the region from START to END with PROPERTIES."
-      (w3m-add-text-properties start end
-			       (append '(face w3m-form) properties)))))
+      (w3m-add-text-properties
+       start end (append (if readonly
+			     '(face w3m-form-inactive w3m-form-readonly t)
+			   '(face w3m-form)
+			   properties))))))
 
 ;;; w3mmee
 ;;
@@ -577,7 +600,8 @@ fid=\\([^/]+\\)/type=\\([^/]+\\)/name=\\([^/]*\\)/id=\\(.*\\)$"
 		    (delete-char 1)
 		    (setq end (point))
 		    (insert "]"))
-		  (w3m-add-face-property start end 'w3m-form)
+		  (w3m-add-face-property
+		   start end (if readonly 'w3m-form-inactive 'w3m-form))
 		  (add-text-properties
 		   start end
 		   `(w3m-form-field-id
@@ -645,7 +669,8 @@ If optional REUSE-FORMS is non-nil, reuse it as `w3m-current-form'."
 			  'application/x-www-form-urlencoded)))
 	      (setq form (w3m-form-new
 			  (or method "get")
-			  (w3m-form-normalize-action action w3m-current-url)
+			  (w3m-form-normalize-action
+			   action (or w3m-current-base-url w3m-current-url))
 			  nil
 			  (if accept-charset
 			      (setq accept-charset
@@ -682,7 +707,7 @@ If optional REUSE-FORMS is non-nil, reuse it as `w3m-current-form'."
 	    (w3m-add-face-property start end 'w3m-form)
 	    (add-text-properties
 	     start (match-beginning 0)
-	     `(w3m-action (w3m-form-input-map ,maps ,usemap))))))
+	     `(w3m-action (w3m-form-input-map ,maps ,id ,usemap))))))
        ((string= tag "/input_alt")
 	(replace-match ""))
        ((string= tag "input_alt")
@@ -706,7 +731,8 @@ If optional REUSE-FORMS is non-nil, reuse it as `w3m-current-form'."
 	  (save-excursion
 	    (search-forward "</input_alt>")
 	    (setq end (match-beginning 0)))
-	  (let ((abs-hseq (or (and (null hseq) 0) (abs hseq))))
+	  (let ((abs-hseq (or (and (null hseq) 0) (abs hseq)))
+		(face (if readonly 'w3m-form-inactive 'w3m-form)))
 	    (setq w3m-max-anchor-sequence
 		  (max abs-hseq w3m-max-anchor-sequence))
 	    (if (eq w3m-type 'w3mmee)
@@ -734,23 +760,26 @@ If optional REUSE-FORMS is non-nil, reuse it as `w3m-current-form'."
 					       (w3m-form-get ,form ,id)
 					       w3m-form-new-session
 					       w3m-form-download)
-		   w3m-anchor-sequence ,abs-hseq))))
+		   w3m-anchor-sequence ,abs-hseq)
+		 readonly)))
 	     ((string= type "reset")
 	      (w3m-form-make-button
 	       start end
 	       `(w3m-form-field-id
 		 ,(format "fid=%d/type=%s/name=%s/id=%d" fid type name id)
 		 w3m-action (w3m-form-reset ,form)
-		 w3m-anchor-sequence ,abs-hseq)))
+		 w3m-anchor-sequence ,abs-hseq)
+	       readonly))
 	     ((string= type "textarea")
 	      (if (eq w3m-type 'w3mmee)
 		  (w3m-form-put form id
 				name
 				(decode-coding-string
-				 (w3m-url-decode-string value
-							w3m-current-coding-system)
+				 (w3m-url-decode-string
+				  value	w3m-current-coding-system)
 				 w3m-output-coding-system))
-		(setq textareas (cons (list textareanumber form id name readonly)
+		(setq textareas (cons (list textareanumber form id
+					    name readonly)
 				      textareas)))
 	      (when w3m-current-url
 		(setq filename (expand-file-name
@@ -761,7 +790,7 @@ If optional REUSE-FORMS is non-nil, reuse it as `w3m-current-form'."
 				w3m-form-textarea-directory))
 		(setq w3m-form-textarea-files
 		      (cons filename w3m-form-textarea-files)))
-	      (w3m-add-face-property start end 'w3m-form)
+	      (w3m-add-face-property start end face)
 	      (add-text-properties
 	       start end
 	       `(w3m-form-field-id
@@ -786,7 +815,7 @@ If optional REUSE-FORMS is non-nil, reuse it as `w3m-current-form'."
 			  t)
 		      (setq selects (cons (list selectnumber form id name)
 					  selects)))
-		(w3m-add-face-property start end 'w3m-form)
+		(w3m-add-face-property start end face)
 		(add-text-properties
 		 start end
 		 `(w3m-form-field-id
@@ -796,9 +825,10 @@ If optional REUSE-FORMS is non-nil, reuse it as `w3m-current-form'."
 					       (w3m-form-get ,form ,id)
 					       w3m-form-new-session
 					       w3m-form-download)
-		   w3m-anchor-sequence ,abs-hseq))))
+		   w3m-anchor-sequence ,abs-hseq
+		   w3m-form-readonly ,readonly))))
 	     ((string= type "password")
-	      (w3m-add-face-property start end 'w3m-form)
+	      (w3m-add-face-property start end face)
 	      (add-text-properties
 	       start end
 	       `(w3m-form-field-id
@@ -816,7 +846,7 @@ If optional REUSE-FORMS is non-nil, reuse it as `w3m-current-form'."
 			      (if checked
 				  (cons value cvalue)
 				cvalue)))
-	      (w3m-add-face-property start end 'w3m-form)
+	      (w3m-add-face-property start end face)
 	      (add-text-properties
 	       start end
 	       `(w3m-form-field-id
@@ -826,24 +856,27 @@ If optional REUSE-FORMS is non-nil, reuse it as `w3m-current-form'."
 					     (w3m-form-get ,form ,id)
 					     w3m-form-new-session
 					     w3m-form-download)
-		 w3m-anchor-sequence ,abs-hseq)))
+		 w3m-anchor-sequence ,abs-hseq
+		 w3m-form-readonly ,readonly)))
 	     ((string= type "radio")
 	      ;; Radio button input, one name has one value
 	      (if checked
 		  (w3m-form-put-by-name form id name value))
-	      (w3m-add-face-property start end 'w3m-form)
+	      (w3m-add-face-property start end face)
 	      (add-text-properties
 	       start end
 	       `(w3m-form-field-id
 		 ,(format "fid=%d/type=%s/name=%s/id=%d" fid type name id)
 		 w3m-action (w3m-form-input-radio ,form ,id ,name ,value)
 		 w3m-submit (w3m-form-submit ,form ,id ,name
-					     (w3m-form-get-by-name ,form ,name)
+					     (w3m-form-get-by-name
+					      ,form ,id ,name)
 					     w3m-form-new-session
 					     w3m-form-download)
-		 w3m-anchor-sequence ,abs-hseq)))
+		 w3m-anchor-sequence ,abs-hseq
+		 w3m-form-readonly ,readonly)))
 	     ((string= type "file")
-	      (w3m-add-face-property start end 'w3m-form)
+	      (w3m-add-face-property start end face)
 	      (add-text-properties
 	       start end
 	       `(w3m-form-field-id
@@ -853,13 +886,14 @@ If optional REUSE-FORMS is non-nil, reuse it as `w3m-current-form'."
 					     (w3m-form-get ,form ,id)
 					     w3m-form-new-session
 					     w3m-form-download)
-		 w3m-anchor-sequence ,abs-hseq)))
+		 w3m-anchor-sequence ,abs-hseq
+		 w3m-form-readonly ,readonly)))
 	     (t
 	      (w3m-form-put form
 			    id
 			    name
 			    (or value (w3m-form-get form id)))
-	      (w3m-add-face-property start end 'w3m-form)
+	      (w3m-add-face-property start end face)
 	      (add-text-properties
 	       start end
 	       `(w3m-form-field-id
@@ -926,8 +960,7 @@ If optional REUSE-FORMS is non-nil, reuse it as `w3m-current-form'."
     (setq w3m-current-forms (if (eq w3m-type 'w3mmee)
 				forms
 			      (mapcar 'cdr
-				      (sort forms (lambda (x y)
-						    (< (car x)(car y)))))))
+				      (sort forms #'car-less-than-car))))
     (w3m-form-resume (or reuse-forms w3m-current-forms))))
 
 (defun w3m-form-replace (string &optional invisible)
@@ -960,7 +993,7 @@ If optional REUSE-FORMS is non-nil, reuse it as `w3m-current-form'."
 (defun w3m-form-input (form id name type width maxlength value)
   (let ((fvalue (w3m-form-get form id)))
     (if (get-text-property (point) 'w3m-form-readonly)
-	(message "READONLY %s: %s" (upcase type) fvalue)
+	(w3m-message "READONLY %s: %s" (upcase type) fvalue)
       (save-excursion
 	(let ((input (save-excursion
 		       (read-from-minibuffer (concat (upcase type) ": ") fvalue)))
@@ -986,13 +1019,15 @@ If optional REUSE-FORMS is non-nil, reuse it as `w3m-current-form'."
       (w3m-form-replace input 'invisible))))
 
 (defun w3m-form-input-checkbox (form id name value)
-  (let ((fvalue (w3m-form-get form id)))
-    (if (member value fvalue)		; already checked
-	(progn
-	  (w3m-form-put form id name (delete value fvalue))
-	  (w3m-form-replace " "))
-      (w3m-form-put form id name (cons value fvalue))
-      (w3m-form-replace "*"))))
+  (if (get-text-property (point) 'w3m-form-readonly)
+      (w3m-message "This form is currently inactive")
+    (let ((fvalue (w3m-form-get form id)))
+      (if (member value fvalue)		; already checked
+	  (progn
+	    (w3m-form-put form id name (delete value fvalue))
+	    (w3m-form-replace " "))
+	(w3m-form-put form id name (cons value fvalue))
+	(w3m-form-replace "*")))))
 
 (defun w3m-form-field-parse (fid)
   (when (and fid
@@ -1005,41 +1040,45 @@ If optional REUSE-FORMS is non-nil, reuse it as `w3m-current-form'."
 	  (match-string 4 fid))))
 
 (defun w3m-form-input-radio (form id name value)
-  (save-excursion
-    (let ((fid (w3m-form-field-parse
-		(get-text-property (point) 'w3m-form-field-id)))
-	  cur-fid)
-      (when fid
-	;; Uncheck all RADIO input having same NAME
-	(goto-char 1)
-	(while (w3m-form-goto-next-field)
-	  (setq cur-fid (w3m-form-field-parse
-			 (get-text-property (point)
-					    'w3m-form-field-id)))
-	  (when (and (string= (nth 0 fid) (nth 0 cur-fid))
-		     (string= (nth 1 fid) (nth 1 cur-fid))
-		     (string= (nth 2 fid) (nth 2 cur-fid)))
-	    (w3m-form-put-by-name
-	     form (string-to-number (nth 3 fid)) (nth 2 fid) nil)
-	    (w3m-form-replace " ")))))) ; erase check
-  ;; Then set this field as checked.
-  (w3m-form-put-by-name form id name value)
-  (w3m-form-replace "*"))
+  (if (get-text-property (point) 'w3m-form-readonly)
+      (w3m-message "This form is currently inactive")
+    (save-excursion
+      (let ((fid (w3m-form-field-parse
+		  (get-text-property (point) 'w3m-form-field-id)))
+	    cur-fid)
+	(when fid
+	  ;; Uncheck all RADIO input having same NAME
+	  (goto-char 1)
+	  (while (w3m-form-goto-next-field)
+	    (setq cur-fid (w3m-form-field-parse
+			   (get-text-property (point)
+					      'w3m-form-field-id)))
+	    (when (and (string= (nth 0 fid) (nth 0 cur-fid))
+		       (string= (nth 1 fid) (nth 1 cur-fid))
+		       (string= (nth 2 fid) (nth 2 cur-fid)))
+	      (w3m-form-put-by-name
+	       form (string-to-number (nth 3 fid)) (nth 2 fid) nil)
+	      (w3m-form-replace " ")))))) ; erase check
+    ;; Then set this field as checked.
+    (w3m-form-put-by-name form id name value)
+    (w3m-form-replace "*")))
 
 (defun w3m-form-input-file (form id name value)
-  (let ((input (save-excursion
-		 (read-file-name "File name: "
-				 (or (cdr (w3m-form-get form id))
-				     "~/")))))
-    (w3m-form-put form id name (cons 'file input))
-    (w3m-form-replace input)))
+  (if (get-text-property (point) 'w3m-form-readonly)
+      (w3m-message "This form is currently inactive")
+    (let ((input (save-excursion
+		   (read-file-name "File name: "
+				   (or (cdr (w3m-form-get form id))
+				       "~/")))))
+      (w3m-form-put form id name (cons 'file input))
+      (w3m-form-replace input))))
 
 ;;; TEXTAREA
 
 (defcustom w3m-form-input-textarea-buffer-lines 10
   "*Buffer lines for form textarea buffer."
   :group 'w3m
-  :type '(integer :size 0))
+  :type 'integer)
 
 (defcustom w3m-form-input-textarea-mode-hook nil
   "*A hook called after w3m-form-input-textarea-mode."
@@ -1112,6 +1151,18 @@ character."
   (define-key w3m-form-input-textarea-map "\C-x\C-s"
     'w3m-form-input-textarea-save))
 
+(defvar w3m-form-input-textarea-org-mode-map nil)
+(unless w3m-form-input-textarea-org-mode-map
+  (setq w3m-form-input-textarea-org-mode-map (make-sparse-keymap))
+  (define-key w3m-form-input-textarea-org-mode-map "\M-#c"
+    'w3m-form-input-textarea-set)
+  (define-key w3m-form-input-textarea-org-mode-map "\M-#q"
+    'w3m-form-input-textarea-exit)
+  (define-key w3m-form-input-textarea-org-mode-map "\M-#k"
+    'w3m-form-input-textarea-exit)
+  (define-key w3m-form-input-textarea-org-mode-map "\M-#s"
+    'w3m-form-input-textarea-save))
+
 (defun w3m-form-input-textarea-filename (url id)
   (condition-case nil
       (concat (md5 (concat url id) nil nil w3m-current-coding-system) ".txt")
@@ -1226,31 +1277,87 @@ Minor mode to edit form textareas of w3m.
 	      (if arg
 		  (> (prefix-numeric-value arg) 0)
 		(not w3m-form-input-textarea-mode)))
-    (run-hooks 'w3m-form-input-textarea-mode-hook)))
+    (run-hooks 'w3m-form-input-textarea-mode-hook))
+  (when (eq major-mode 'org-mode)
+    (add-to-list 'minor-mode-overriding-map-alist
+		 (cons 'w3m-form-input-textarea-mode
+		       w3m-form-input-textarea-org-mode-map))))
 
-(defun w3m-form-input-textarea-mode-setup (caller-buffer)
-  (funcall (if (functionp w3m-form-textarea-edit-mode)
-	       w3m-form-textarea-edit-mode
-	     (or (when (buffer-live-p caller-buffer)
-		   (with-current-buffer caller-buffer
-		     (save-match-data
-		       (catch 'found-mode
-			 (dolist (elem w3m-form-textarea-edit-mode)
-			   (when (if (stringp (car elem))
-				     (string-match (car elem)
-						   w3m-current-url)
-				   (if (functionp (car elem))
-				       (funcall (car elem))
-				     (eval (car elem))))
-			     (throw 'found-mode (cdr elem))))))))
-		 'text-mode)))
-  (w3m-form-input-textarea-mode 1)
-  (message "%s"
-	   (substitute-command-keys "Type \
+(defvar w3m-form-textarea-use-org-mode-p nil
+  "Use org-mode for editing textareas when non-nil")
+
+(defun w3m-form-textarea-toggle-major-mode (&optional arg)
+  "Toggle editing textareas in Org-mode.
+When off, textareas are edited in text-mode, otherwise in
+org-mode. With prefix argument ARG, use org-mode if ARG is
+positive, otherwise text-mode."
+  (interactive "P")
+  (setq w3m-form-textarea-use-org-mode-p
+	(if (null arg)
+	    (not w3m-form-textarea-use-org-mode-p)
+	  (> (prefix-numeric-value arg) 0)))
+  (message "Edit textarea in Org-mode %s"
+	   (if w3m-form-textarea-use-org-mode-p "enabled" "disabled")))
+
+(eval-when-compile (require 'outline))
+(w3m-static-if (fboundp 'outline-show-all)
+    ;; Emacs >=25
+    (progn
+      (autoload 'outline-show-all "outline" nil t)
+      (add-hook 'w3m-form-input-textarea-mode-hook
+		(lambda ()
+		  (and (eq major-mode 'org-mode) (outline-show-all)))))
+  (autoload 'show-all "outline" nil t)
+  (add-hook 'w3m-form-input-textarea-mode-hook
+	    (lambda ()
+	      (and (eq major-mode 'org-mode) (show-all)))))
+
+(defvar view-mode-map)
+(defun w3m-form-input-textarea-mode-setup (caller-buffer readonly)
+  (funcall (or (and readonly
+		    'view-mode)
+	       (and w3m-form-textarea-use-org-mode-p
+		    'org-mode)
+	       (and (functionp w3m-form-textarea-edit-mode)
+		    w3m-form-textarea-edit-mode)
+	       (when (buffer-live-p caller-buffer)
+		 (with-current-buffer caller-buffer
+		   (save-match-data
+		     (catch 'found-mode
+		       (dolist (elem w3m-form-textarea-edit-mode)
+			 (when (if (stringp (car elem))
+				   (string-match (car elem)
+						 w3m-current-url)
+				 (if (functionp (car elem))
+				     (funcall (car elem))
+				   (eval (car elem))))
+			   (throw 'found-mode (cdr elem))))))))
+	       'text-mode))
+  (if readonly
+      (let ((keymap (copy-keymap view-mode-map)))
+	(set (make-local-variable 'minor-mode-map-alist)
+	     (list (cons 'view-mode keymap)))
+	(dolist (key (nconc (where-is-internal 'w3m-form-input-textarea-set
+					       w3m-form-input-textarea-map)
+			    (where-is-internal 'w3m-form-input-textarea-exit
+					       w3m-form-input-textarea-map)))
+	  (define-key keymap key 'w3m-form-input-textarea-exit))
+	(dolist (command '(View-exit View-exit-and-edit View-kill-and-leave
+				     View-leave View-quit View-quit-all))
+	  (substitute-key-definition command
+				     'w3m-form-input-textarea-exit keymap))
+	(w3m-message (substitute-command-keys "READONLY TEXT; type \
+`\\<w3m-form-input-textarea-map>\\[w3m-form-input-textarea-exit]' \
+to quit textarea")))
+    (w3m-form-input-textarea-mode 1)
+    (w3m-message "%s"
+		 (if readonly "\
+This text is not editable because the form has disabled or readonly attribute"
+		   (substitute-command-keys "Type \
 `\\<w3m-form-input-textarea-map>\\[w3m-form-input-textarea-set]' to exit \
 textarea, or type \
 `\\<w3m-form-input-textarea-map>\\[w3m-form-input-textarea-exit]' to quit \
-textarea")))
+textarea")))))
 
 (eval-and-compile
   (defalias 'w3m-same-window-p
@@ -1282,6 +1389,10 @@ selected rather than \(as usual\) some other window.  See
 			 (throw 'found t)))))))))
       'same-window-p)))
 
+(eval-when-compile
+  (unless (fboundp 'split-window-sensibly)
+    (defalias 'split-window-sensibly 'ignore)))
+
 (defun w3m-form-input-textarea (form hseq)
   (let* ((info  (w3m-form-textarea-info))
 	 (value (w3m-form-get form (car info)))
@@ -1320,6 +1431,7 @@ selected rather than \(as usual\) some other window.  See
 	(with-temp-buffer
 	  (let ((buffer-file-coding-system w3m-form-textarea-file-coding-system)
 		(coding-system-for-read w3m-form-textarea-file-coding-system))
+
 	    (insert-file-contents file))
 	  (let ((before (buffer-string)))
 	    (when (unless (w3m-form-textarea-same-check value before)
@@ -1338,10 +1450,12 @@ selected rather than \(as usual\) some other window.  See
       (with-current-buffer buffer
 	(insert value)
 	(set-buffer-modified-p nil)
-	(when readonly (setq buffer-read-only t))
+	(when readonly
+	  (set-buffer-modified-p nil)
+	  (setq buffer-read-only t))
 	(goto-char (point-min))
 	(forward-line (1- (nth 2 info)))
-	(w3m-form-input-textarea-mode-setup w3mbuffer)
+	(w3m-form-input-textarea-mode-setup w3mbuffer readonly)
 	(setq w3m-form-input-textarea-form form
 	      w3m-form-input-textarea-hseq hseq
 	      w3m-form-input-textarea-buffer w3mbuffer
@@ -1357,12 +1471,18 @@ selected rather than \(as usual\) some other window.  See
       ;; Use the whole current window for the textarea when a user added
       ;; the buffer name "*w3m form textarea*" to `same-window-buffer-names'
       ;; (that is available only in Emacs).
-      ;; cf. http://article.gmane.org/gmane.emacs.w3m/7797
+      ;; cf.
+      ;; <https://lists.gnu.org/archive/html/help-gnu-emacs/2008-11/msg00551.html>
       (unless (w3m-same-window-p (buffer-name (if (consp buffer)
 						  (cdr buffer)
 						buffer)))
 	(condition-case nil
-	    (split-window cur-win (if (> size 0) size window-min-height))
+	    (if (and w3m-form-textarea-use-org-mode-p
+		     (fboundp 'split-window-sensibly) ;; Emacs >= 23
+		     (not (eq (symbol-function 'split-window-sensibly)
+			      'ignore)))
+		(split-window-sensibly cur-win)
+	      (split-window cur-win (if (> size 0) size window-min-height)))
 	  (error
 	   (delete-other-windows)
 	   (split-window cur-win (- (window-height cur-win)
@@ -1477,7 +1597,7 @@ selected rather than \(as usual\) some other window.  See
 (defcustom w3m-form-input-select-buffer-lines 10
   "*Buffer lines for form select buffer."
   :group 'w3m
-  :type '(integer :size 0))
+  :type 'integer)
 
 (defcustom w3m-form-input-select-mode-hook nil
   "*A hook called after w3m-form-input-select-mode."
@@ -1585,81 +1705,83 @@ selected rather than \(as usual\) some other window.  See
   (w3m-run-mode-hooks 'w3m-form-input-select-mode-hook))
 
 (defun w3m-form-input-select (form id name)
-  (let* ((value (w3m-form-get form id))
-	 (cur-win (selected-window))
-	 (wincfg (current-window-configuration))
-	 (urlid (format "%s:%s:%d" w3m-current-url name id))
-	 (w3mbuffer (current-buffer))
-	 (point (point))
-	 (size (min
-		(- (window-height cur-win)
-		   window-min-height 1)
-		(- (window-height cur-win)
-		   (max window-min-height
-			(1+ w3m-form-input-select-buffer-lines)))))
-	 buffer cur pos)
-    (setq buffer
-	  (catch 'detect-buffer
-	    (save-current-buffer
-	      (dolist (buffer (buffer-list))
-		(set-buffer buffer)
-		(when (and w3m-form-input-select-buffer
-			   (eq w3m-form-input-select-buffer w3mbuffer)
-			   (string= w3m-form-input-select-urlid urlid))
-		  (throw 'detect-buffer (cons t buffer)))))
-	    (generate-new-buffer "*w3m form select*")))
-    (unless (consp buffer)
-      (with-current-buffer buffer
-	(setq w3m-form-input-select-form form)
-	(setq w3m-form-input-select-name name)
-	(setq w3m-form-input-select-id id)
-	(setq w3m-form-input-select-buffer w3mbuffer)
-	(setq w3m-form-input-select-point point)
-	(setq w3m-form-input-select-candidates value)
-	(setq w3m-form-input-select-wincfg wincfg)
-	(setq w3m-form-input-select-urlid urlid)
-	(when value
-	  (setq cur (car value))
-	  (setq value (cdr value))
-	  (dolist (candidate value)
-	    (setq pos (point))
-	    (insert (if (zerop (length (cdr candidate)))
-			" "		; "" -> " "
-		      (cdr candidate)))
-	    (add-text-properties pos (point)
-				 (list 'w3m-form-select-value (car candidate)
-				       'mouse-face w3m-form-mouse-face))
-	    (insert "\n")))
-	(goto-char (point-min))
-	(while (and (not (eobp))
-		    (not (equal cur
-				(get-text-property (point)
-						   'w3m-form-select-value))))
-	  (goto-char (next-single-property-change (point)
-						  'w3m-form-select-value)))
-	(set-buffer-modified-p nil)
-	(beginning-of-line)
-	(w3m-form-input-select-mode)))
-    (if (and (consp buffer)
-	     (get-buffer-window (cdr buffer)))
-	;; same frame only
-	(select-window (get-buffer-window (cdr buffer)))
-      (condition-case nil
-	  (split-window cur-win (if (> size 0) size window-min-height))
-	(error
-	 (delete-other-windows)
-	 (split-window cur-win (- (window-height cur-win)
-				  w3m-form-input-select-buffer-lines))))
-      (select-window (next-window))
-      (let ((pop-up-windows nil))
-	(switch-to-buffer (if (consp buffer) (cdr buffer) buffer))))))
+  (if (get-text-property (point) 'w3m-form-readonly)
+      (w3m-message "This form is currently inactive")
+    (let* ((value (w3m-form-get form id))
+	   (cur-win (selected-window))
+	   (wincfg (current-window-configuration))
+	   (urlid (format "%s:%s:%d" w3m-current-url name id))
+	   (w3mbuffer (current-buffer))
+	   (point (point))
+	   (size (min
+		  (- (window-height cur-win)
+		     window-min-height 1)
+		  (- (window-height cur-win)
+		     (max window-min-height
+			  (1+ w3m-form-input-select-buffer-lines)))))
+	   buffer cur pos)
+      (setq buffer
+	    (catch 'detect-buffer
+	      (save-current-buffer
+		(dolist (buffer (buffer-list))
+		  (set-buffer buffer)
+		  (when (and w3m-form-input-select-buffer
+			     (eq w3m-form-input-select-buffer w3mbuffer)
+			     (string= w3m-form-input-select-urlid urlid))
+		    (throw 'detect-buffer (cons t buffer)))))
+	      (generate-new-buffer "*w3m form select*")))
+      (unless (consp buffer)
+	(with-current-buffer buffer
+	  (setq w3m-form-input-select-form form)
+	  (setq w3m-form-input-select-name name)
+	  (setq w3m-form-input-select-id id)
+	  (setq w3m-form-input-select-buffer w3mbuffer)
+	  (setq w3m-form-input-select-point point)
+	  (setq w3m-form-input-select-candidates value)
+	  (setq w3m-form-input-select-wincfg wincfg)
+	  (setq w3m-form-input-select-urlid urlid)
+	  (when value
+	    (setq cur (car value))
+	    (setq value (cdr value))
+	    (dolist (candidate value)
+	      (setq pos (point))
+	      (insert (if (zerop (length (cdr candidate)))
+			  " "		; "" -> " "
+			(cdr candidate)))
+	      (add-text-properties pos (point)
+				   (list 'w3m-form-select-value (car candidate)
+					 'mouse-face w3m-form-mouse-face))
+	      (insert "\n")))
+	  (goto-char (point-min))
+	  (while (and (not (eobp))
+		      (not (equal cur
+				  (get-text-property (point)
+						     'w3m-form-select-value))))
+	    (goto-char (next-single-property-change (point)
+						    'w3m-form-select-value)))
+	  (set-buffer-modified-p nil)
+	  (beginning-of-line)
+	  (w3m-form-input-select-mode)))
+      (if (and (consp buffer)
+	       (get-buffer-window (cdr buffer)))
+	  ;; same frame only
+	  (select-window (get-buffer-window (cdr buffer)))
+	(condition-case nil
+	    (split-window cur-win (if (> size 0) size window-min-height))
+	  (error
+	   (delete-other-windows)
+	   (split-window cur-win (- (window-height cur-win)
+				    w3m-form-input-select-buffer-lines))))
+	(select-window (next-window))
+	(let ((pop-up-windows nil))
+	  (switch-to-buffer (if (consp buffer) (cdr buffer) buffer)))))))
 
 ;;; MAP
 
 (defcustom w3m-form-input-map-buffer-lines 10
   "*Buffer lines for form select map buffer."
   :group 'w3m
-  :type '(integer :size 0))
+  :type 'integer)
 
 (defcustom w3m-form-input-map-mode-hook nil
   "*A hook called after w3m-form-input-map-mode."
@@ -1752,8 +1874,8 @@ selected rather than \(as usual\) some other window.  See
   (use-local-map w3m-form-input-map-keymap)
   (w3m-run-mode-hooks 'w3m-form-input-map-mode-hook))
 
-(defun w3m-form-input-map (form name)
-  (let* ((value (w3m-form-get-by-name form name))
+(defun w3m-form-input-map (form id name)
+  (let* ((value (w3m-form-get-by-name form id name))
 	 (urlname (format "%s:%s" w3m-current-url name))
 	 (cur-win (selected-window))
 	 (wincfg (current-window-configuration))
@@ -1824,6 +1946,17 @@ selected rather than \(as usual\) some other window.  See
 	  (setq files (cons file files))))
       files)))
 
+(defvar w3m-form-last-position nil)
+
+(defun w3m-form-restore-last-position ()
+  "Restore positions after fontifying a page containing forms."
+  (when (> (point-max) (nth 0 w3m-form-last-position))
+    (set-window-start nil (goto-char (nth 0 w3m-form-last-position)))
+    (set-window-hscroll nil (nth 1 w3m-form-last-position))
+    (forward-line (max 0 (1- (nth 2 w3m-form-last-position))))
+    (move-to-column (nth 3 w3m-form-last-position)))
+  (remove-hook 'w3m-fontify-after-hook 'w3m-form-restore-last-position))
+
 (defun w3m-form-submit (form &optional id name value new-session download)
   (if (w3m-anchor (point))
       ;; cf SA17565
@@ -1831,11 +1964,14 @@ selected rather than \(as usual\) some other window.  See
     (when (and id name
 	       (> (length name) 0))
       (w3m-form-put form id name value))
-    (let* ((orig-url w3m-current-url)
-	   (url (or (w3m-form-action form)
-		    (if (string-match "\\?" w3m-current-url)
-			(substring w3m-current-url 0 (match-beginning 0))
-		      w3m-current-url))))
+    (let ((orig-url w3m-current-url)
+	  (url (or (w3m-form-action form)
+		   (if (string-match "\\?" w3m-current-url)
+		       (substring w3m-current-url 0 (match-beginning 0))
+		     w3m-current-url))))
+      (setq w3m-form-last-position (list (window-start) (window-hscroll)
+					 (count-lines (window-start) (point))
+					 (current-column)))
       (setq w3m-form-textarea-post-files
 	    (w3m-form-submit-get-textarea-files form))
       (cond ((and (not (string= url orig-url))
@@ -1849,10 +1985,12 @@ selected rather than \(as usual\) some other window.  See
 		 ;; we use the `post' method according to the proposal
 		 ;; of RFC2070.
 		 (eq 'multipart/form-data (w3m-form-enctype form)))
-	     (if download 
+	     (if download
 		 (funcall 'w3m-download
 			  url nil nil nil
 			  (w3m-form-make-form-data form))
+	       (add-hook 'w3m-fontify-after-hook
+			 'w3m-form-restore-last-position)
 	       (funcall (if new-session
 			    'w3m-goto-url-new-session
 			  'w3m-goto-url)
@@ -1860,13 +1998,17 @@ selected rather than \(as usual\) some other window.  See
 			(w3m-form-make-form-data form)
 			w3m-current-url)))
 	    ((eq 'get (w3m-form-method form))
-	     (funcall (if download 
+	     (funcall (if download
 			  'w3m-download
+			(add-hook 'w3m-fontify-after-hook
+				  'w3m-form-restore-last-position)
 			(if new-session
 			    'w3m-goto-url-new-session
 			  'w3m-goto-url))
-		      (concat (w3m-url-strip-query url) 
-			      "?" (w3m-form-make-form-data form))))
+		      (let ((query (w3m-form-make-form-data form)))
+			(if (zerop (length query))
+			    url
+			  (concat (w3m-url-strip-query url) "?" query)))))
 	    (t
 	     (w3m-message "This form's method has not been supported: %s"
 			  (let (print-level print-length)
@@ -1889,6 +2031,117 @@ selected rather than \(as usual\) some other window.  See
 	(goto-char (or (w3m-form-real-reset form (w3m-action pos))
 		       (next-single-property-change pos 'w3m-action)))))))
 
+(defun w3m-form-expand-form ()
+  "Expand the form at point so as to show the contents fully.
+`w3m-show-form-hint' uses this function."
+  (unless (get-text-property (point) 'w3m-form-expanded)
+    (let ((inhibit-read-only t)
+	  (act (w3m-action))
+	  (mod (buffer-modified-p))
+	  value from pt to keymap)
+      (when (cond ((eq (car act) 'w3m-form-input)
+		   (setq value (w3m-form-get (nth 1 act) (nth 2 act))))
+		  ((eq (car act) 'w3m-form-input-textarea)
+		   (setq value (w3m-form-get (nth 1 act)
+					     (car (w3m-form-textarea-info))))))
+	(if (bobp)
+	    (setq from (point))
+	  (if (or (get-text-property (1- (point)) 'w3m-action)
+		  (and (setq pt (previous-single-property-change
+				 (1- (point)) 'w3m-action))
+		       (equal (get-text-property
+			       (setq pt (max (1- pt) (point-min))) 'w3m-action)
+			      act)))
+	      (while (and (not from)
+			  (setq from (previous-single-property-change
+				      (or pt (point)) 'w3m-action)))
+		(when (and (setq pt (previous-single-property-change
+				     from 'w3m-action))
+			   (equal (get-text-property
+				   (max (1- pt) (point-min)) 'w3m-action)
+				  act))
+		  (setq from nil)))
+	    (setq from (point))))
+	(setq pt nil)
+	(while (and (not to)
+		    (setq to (text-property-any (or pt (point)) (point-max)
+						'w3m-action nil)))
+	  (when (and (setq pt (next-single-property-change to 'w3m-action))
+		     (equal (get-text-property pt 'w3m-action) act))
+	    (setq to nil)))
+	(setq keymap (make-sparse-keymap))
+	(define-key keymap "c" `(lambda nil (interactive) (kill-new ,value)))
+	(w3m-static-when (featurep 'emacs)
+	  (define-key keymap [mouse-1] 'ignore)
+	  (define-key keymap [drag-mouse-1] 'ignore))
+	(if (<= (length (replace-regexp-in-string "[\t\n ]+" "" value))
+		(length (replace-regexp-in-string
+			 "[\t\n ]+" ""
+			 (replace-regexp-in-string
+			  "][\t\n ]*\\[" ""
+			  (buffer-substring-no-properties from to)))))
+	    (put-text-property from to 'keymap keymap)
+	  (setq pt (cons (current-column)
+			 (- (count-lines from (point))
+			    (if (= from (point)) 0 1))))
+	  (goto-char to)
+	  (add-text-properties
+	   from to (list 'rear-nonsticky nil 'keymap keymap))
+	  (insert-and-inherit value)
+	  (put-text-property to (point) 'w3m-form-expanded
+			     (list from (- (point) (- to from))
+				   (buffer-substring from to)))
+	  (dolist (o (overlays-at from))
+	    (when (= (overlay-start o) from)
+	      (move-overlay o to (point))))
+	  (delete-region from to)
+	  (goto-char from)
+	  (forward-line (cdr pt))
+	  (move-to-column (car pt))
+	  (deactivate-mark)
+	  (set-buffer-modified-p mod))))))
+
+(defun w3m-form-unexpand-form ()
+  "Unexpand expanded form not being in the current line.
+`w3m-show-form-hint' uses this function."
+  (unless (or (get-text-property (point) 'w3m-form-expanded)
+	      (< (next-single-property-change
+		  (point-at-bol) 'w3m-form-expanded nil (point-at-eol))
+		 (point-at-eol))
+	      (and (eq (char-after (point-at-bol)) ?\])
+		   (get-text-property (max (1- (point-at-bol)) (point-min))
+				      'w3m-form-expanded)))
+    (let ((pt (text-property-not-all (point-min) (point-max)
+				     'w3m-form-expanded nil)))
+      (when pt
+	(let* ((inhibit-read-only t)
+	       (expanded (when pt (get-text-property pt 'w3m-form-expanded)))
+	       (from (car expanded))
+	       (to (nth 1 expanded))
+	       (value (nth 2 expanded))
+	       (mod (buffer-modified-p)))
+	  (add-text-properties
+	   from to '(keymap nil rear-nonsticky t w3m-form-expanded nil))
+	  (setq pt (cons (current-column)
+			 (if (< (point) from)
+			     (- 1 (count-lines (point) from))
+			   (max 1 (+ (count-lines to (point))
+				     (if (= to (point))
+					 1
+				       (if (bolp) 0 -1)))))))
+	  (goto-char to)
+	  (insert value)
+	  (dolist (o (overlays-at from))
+	    (when (= (overlay-start o) from)
+	      (move-overlay o to (point))))
+	  (unless (< (cdr pt) 0) (forward-line (cdr pt)))
+	  (delete-region from to)
+	  (when (< (cdr pt) 0)
+	    (goto-char from)
+	    (forward-line (cdr pt)))
+	  (move-to-column (car pt))
+	  (deactivate-mark)
+	  (set-buffer-modified-p mod))))))
 
 (provide 'w3m-form)
 
