@@ -1,6 +1,6 @@
 ;;; w3m-proc.el --- Functions and macros to control sub-processes
 
-;; Copyright (C) 2001-2005, 2007-2010, 2012, 2013, 2016
+;; Copyright (C) 2001-2005, 2007-2010, 2012, 2013, 2016-2018
 ;; TSUCHIYA Masatoshi <tsuchiya@namazu.org>
 
 ;; Authors: TSUCHIYA Masatoshi <tsuchiya@namazu.org>,
@@ -44,21 +44,23 @@
 (require 'w3m-util)
 
 (eval-when-compile
-  ;; Variable(s) which are used in the following inline functions.
-  ;; They should be defined in the other module at run-time.
-  (defvar w3m-current-url)
-  (defvar w3m-current-buffer)
-  (defvar w3m-current-process)
-  (defvar w3m-profile-directory)
-  (defvar w3m-terminal-coding-system)
+  ;; External functions and variables used in this module,
+  ;; and should be defined in the other module at run-time.
+  (autoload 'w3m-idle-images-show-unqueue "w3m")
+  (defvar w3m-async-exec)
+  (defvar w3m-clear-display-while-reading)
   (defvar w3m-command)
   (defvar w3m-command-arguments)
   (defvar w3m-command-environment)
-  (defvar w3m-async-exec)
+  (defvar w3m-current-buffer)
+  (defvar w3m-current-process)
+  (defvar w3m-current-title)
+  (defvar w3m-current-url)
   (defvar w3m-process-connection-type)
   (defvar w3m-process-modeline-format)
-  (defvar w3m-work-buffer-list)
-  (autoload 'w3m-idle-images-show-unqueue "w3m"))
+  (defvar w3m-profile-directory)
+  (defvar w3m-terminal-coding-system)
+  (defvar w3m-work-buffer-list))
 
 (defvar w3m-process-inhibit-quit t
   "`w3m-process-sentinel' binds `inhibit-quit' according to this variable.")
@@ -170,9 +172,9 @@
   `(aref ,handler 3))
 
 (defun w3m-process-push (handler command arguments)
-  "Generate a new `w3m-process' object which is provided by HANDLER,
-ARGUMENTS and this buffer, regist it to `w3m-process-queue', and
-return it."
+  "Generate a new `w3m-process' object.
+The process will be provided by HANDLER, ARGUMENTS and this buffer,
+registered to `w3m-process-queue', and returned."
   (let ((x (assoc (cons command arguments) w3m-process-queue)))
     (unless x
       (setq x (w3m-process-new command arguments (current-buffer)))
@@ -236,7 +238,7 @@ generated asynchronous process is ignored.  Otherwise,
 (defun w3m-process-start-queued-processes ()
   "Start a process which is registerd in `w3m-process-queue' if the
 number of current working processes is less than `w3m-process-max'."
-  (w3m-process-kill-stray-processes)
+  ;; Run `w3m-process-kill-stray-processes' in advance.
   (let ((num 0))
     (catch 'last
       (dolist (obj (reverse w3m-process-queue))
@@ -292,7 +294,22 @@ which have no handler."
   (when (buffer-name buffer)
     (with-current-buffer buffer
       (setq w3m-current-process nil)))
-  (w3m-process-start-queued-processes)
+  (w3m-process-kill-stray-processes)
+  (if w3m-process-queue
+      (w3m-process-start-queued-processes)
+    (when w3m-clear-display-while-reading
+      (let ((pt (point)))
+	(goto-char (point-min))
+	(if (re-search-forward "\n*\\( *\\)Reading [^\n]+\\(\\.\\.\\.\\)"
+			       nil t)
+	    (let ((inhibit-read-only t))
+	      (delete-region (match-beginning 2) (point-max))
+	      (insert "\n\n" (match-string 1) "Operation aborted by user.")
+	      (delete-region 1 (min 3 (match-beginning 1)))
+	      (set-buffer-modified-p nil)
+	      (setq w3m-current-url nil
+		    w3m-current-title nil))
+	  (goto-char pt)))))
   (w3m-force-window-update-later buffer))
 
 (defun w3m-process-shutdown ()
@@ -513,10 +530,7 @@ evaluated in a temporary buffer."
    ((not exit-status)
     (setq w3m-process-exit-status nil))
    (t
-    (setq w3m-process-exit-status
-	  (w3m-static-if (featurep 'emacs)
-	      (string-make-multibyte (format "%s" exit-status))
-	    (format "%s" exit-status)))
+    (setq w3m-process-exit-status (format "%s" exit-status))
     nil)))
 
 (defvar w3m-process-background nil
@@ -576,6 +590,7 @@ evaluated in a temporary buffer."
 		(throw 'last nil)))))
       (delete-process process)
       (unless ignore-queue
+	(w3m-process-kill-stray-processes)
 	(w3m-process-start-queued-processes)))))
 
 (defun w3m-process-filter (process string)
