@@ -1,6 +1,6 @@
 ;;; w3m-search.el --- functions convenient to access web search engines
 
-;; Copyright (C) 2001--2012 TSUCHIYA Masatoshi <tsuchiya@namazu.org>
+;; Copyright (C) 2001--2012, 2017, 2018 TSUCHIYA Masatoshi <tsuchiya@namazu.org>
 
 ;; Authors: Keisuke Nishida    <kxn30@po.cwru.edu>,
 ;;          Shun-ichi GOTO     <gotoh@taiyo.co.jp>,
@@ -48,186 +48,173 @@
 		    (and (not ja) (w3m-find-coding-system 'utf-8)))))
     `(,@(if ja
 	    '(("yahoo"
-	       "http://search.yahoo.co.jp/bin/search?p=%s"
+	       "https://search.yahoo.co.jp/bin/search?p=%s"
 	       euc-japan)
 	      ("yahoo-en"
-	       "http://search.yahoo.com/bin/search?p=%s"))
+	       "https://search.yahoo.com/bin/search?p=%s"))
 	  '(("yahoo"
-	     "http://search.yahoo.com/bin/search?p=%s")
+	     "https://search.yahoo.com/bin/search?p=%s")
 	    ("yahoo-ja"
-	     "http://search.yahoo.co.jp/bin/search?p=%s"
+	     "https://search.yahoo.co.jp/bin/search?p=%s"
 	     euc-japan)))
-      ("alc" "http://eow.alc.co.jp/%s/UTF-8/" utf-8)
+      ("alc" "https://eow.alc.co.jp/%s/UTF-8/" utf-8)
       ,@(cond
 	 ((and ja utf-8)
 	  '(("blog"
-	     "http://blogsearch.google.com/blogsearch?q=%s&hl=ja&lr=lang_ja&oe=utf-8&ie=utf-8"
+	     "https://blogsearch.google.com/blogsearch?q=%s&hl=ja&lr=lang_ja&oe=utf-8&ie=utf-8"
 	     utf-8)
 	    ("blog-en"
-	     "http://blogsearch.google.com/blogsearch?q=%s&hl=en&oe=utf-8&ie=utf-8"
+	     "https://blogsearch.google.com/blogsearch?q=%s&hl=en&oe=utf-8&ie=utf-8"
 	     utf-8)))
 	 (ja
 	  '(("blog"
-	     "http://blogsearch.google.com/blogsearch?q=%s&hl=ja&lr=lang_ja&ie=Shift_JIS&oe=Shift_JIS"
+	     "https://blogsearch.google.com/blogsearch?q=%s&hl=ja&lr=lang_ja&ie=Shift_JIS&oe=Shift_JIS"
 	     shift_jis)
 	    ("blog-en"
-	     "http://blogsearch.google.com/blogsearch?q=%s&hl=en")))
+	     "https://blogsearch.google.com/blogsearch?q=%s&hl=en")))
 	 (utf-8
 	  '(("blog"
-	     "http://blogsearch.google.com/blogsearch?q=%s&oe=utf-8&ie=utf-8"
+	     "https://blogsearch.google.com/blogsearch?q=%s&oe=utf-8&ie=utf-8"
 	     utf-8)
 	    ("blog-en"
-	     "http://blogsearch.google.com/blogsearch?q=%s&hl=en&oe=utf-8&ie=utf-8"
+	     "https://blogsearch.google.com/blogsearch?q=%s&hl=en&oe=utf-8&ie=utf-8"
 	     utf-8)))
 	 (t
 	  '(("blog"
-	     "http://blogsearch.google.com/blogsearch?q=%s")
+	     "https://blogsearch.google.com/blogsearch?q=%s")
 	    ("blog-ja"
-	     "http://blogsearch.google.com/blogsearch?q=%s&lr=lang_ja&ie=Shift_JIS&oe=Shift_JIS"
+	     "https://blogsearch.google.com/blogsearch?q=%s&lr=lang_ja&ie=Shift_JIS&oe=Shift_JIS"
 	     shift_jis))))
       ,@(cond
 	 ((and ja utf-8)
 	  '(("google"
-	     "http://www.google.com/search?q=%s&hl=ja&lr=lang_ja&ie=utf-8&oe=utf-8"
+	     "https://www.google.com/search?q=%s&hl=ja&lr=lang_ja&ie=utf-8&oe=utf-8"
 	     utf-8)
 	    ("google-en"
-	     "http://www.google.com/search?q=%s&hl=en&ie=utf-8&oe=utf-8"
+	     "https://www.google.com/search?q=%s&hl=en&ie=utf-8&oe=utf-8"
 	     utf-8)))
 	 (ja
 	  '(("google"
-	     "http://www.google.com/search?q=%s&hl=ja&lr=lang_ja&ie=Shift_JIS&oe=Shift_JIS"
+	     "https://www.google.com/search?q=%s&hl=ja&lr=lang_ja&ie=Shift_JIS&oe=Shift_JIS"
 	     shift_jis)
 	    ("google-en"
-	     "http://www.google.com/search?q=%s&hl=en")))
+	     "https://www.google.com/search?q=%s&hl=en")))
 	 (utf-8
 	  '(("google"
-	     "http://www.google.com/search?q=%s&ie=utf-8&oe=utf-8"
+	     "https://www.google.com/search?q=%s&ie=utf-8&oe=utf-8"
 	     utf-8)
 	    ("google-en"
-	     "http://www.google.com/search?q=%s&hl=en&ie=utf-8&oe=utf-8"
+	     "https://www.google.com/search?q=%s&hl=en&ie=utf-8&oe=utf-8"
 	     utf-8)))
 	 (t
 	  '(("google"
-	     "http://www.google.com/search?q=%s")
+	     "https://www.google.com/search?q=%s")
 	    ("google-ja"
-	     "http://www.google.com/search?q=%s&hl=ja&lr=lang_ja&ie=Shift_JIS&oe=Shift_JIS"
+	     "https://www.google.com/search?q=%s&hl=ja&lr=lang_ja&ie=Shift_JIS&oe=Shift_JIS"
 	     shift_jis))))
       ,@(cond
 	 ((and ja utf-8)
 	  '(("google news"
-	     "http://news.google.co.jp/news?q=%s&hl=ja&ie=utf-8&oe=utf-8"
+	     "https://news.google.co.jp/news?q=%s&hl=ja&ie=utf-8&oe=utf-8"
 	     utf-8)
 	    ("google news-en"
-	     "http://news.google.com/news?q=%s&hl=en")))
+	     "https://news.google.com/news?q=%s&hl=en")))
 	 (ja
 	  '(("google news"
-	     "http://news.google.co.jp/news?q=%s&hl=ja&ie=Shift_JIS&oe=Shift_JIS"
+	     "https://news.google.co.jp/news?q=%s&hl=ja&ie=Shift_JIS&oe=Shift_JIS"
 	     shift_jis)
 	    ("google news-en"
-	     "http://news.google.com/news?q=%s&hl=en")))
+	     "https://news.google.com/news?q=%s&hl=en")))
 	 (utf-8
 	  '(("google news"
-	     "http://news.google.com/news?q=%s&ie=utf-8&oe=utf-8"
+	     "https://news.google.com/news?q=%s&ie=utf-8&oe=utf-8"
 	     utf-8)
 	    ("google news-en"
-	     "http://news.google.com/news?q=%s&hl=en&ie=utf-8&oe=utf-8")))
+	     "https://news.google.com/news?q=%s&hl=en&ie=utf-8&oe=utf-8")))
 	 (t
 	  '(("google news"
-	     "http://news.google.com/news?q=%s")
+	     "https://news.google.com/news?q=%s")
 	    ("google news-ja"
-	     "http://news.google.co.jp/news?q=%s&hl=ja&ie=Shift_JIS&oe=Shift_JIS"
+	     "https://news.google.co.jp/news?q=%s&hl=ja&ie=Shift_JIS&oe=Shift_JIS"
 	     shift_jis))))
-      ("google groups"
-       "http://groups.google.com/groups?q=%s")
-      ,@(if ja
-	    '(("All the Web"
-	       "http://www.alltheweb.com/search?q=%s&web&_sb_lang=ja&cs=euc-jp"
-	       euc-japan)
-	      ("All the Web-en"
-	       "http://www.alltheweb.com/search?q=%s&web&_sb_lang=en"))
-	  '(("All the Web"
-	     "http://www.alltheweb.com/search?q=%s&web&_sb_lang=en")
-	    ("All the Web-ja"
-	     "http://www.alltheweb.com/search?q=%s&web&_sb_lang=ja&cs=euc-jp"
-	     euc-japan)))
       ,@(if ja
 	    '(("technorati"
-	       "http://www.technorati.jp/search/search.html?query=%s&language=ja"
+	       "https://www.technorati.jp/search/search.html?query=%s&language=ja"
 	       utf-8)
 	      ("technorati-en"
-	       "http://www.technorati.com/search/%s"
+	       "https://www.technorati.com/search/%s"
 	       utf-8))
 	  '(("technorati"
-	     "http://www.technorati.com/search/%s"
+	     "https://www.technorati.com/search/%s"
 	     utf-8)
 	    ("technorati-ja"
-	     "http://www.technorati.jp/search/search.html?query=%s&language=ja"
+	     "https://www.technorati.jp/search/search.html?query=%s&language=ja"
 	     utf-8)))
       ("technorati-tag"
-       "http://www.technorati.com/tag/%s"
+       "https://www.technorati.com/tag/%s"
        utf-8)
       ("goo-ja"
-       "http://search.goo.ne.jp/web.jsp?MT=%s"
+       "https://search.goo.ne.jp/web.jsp?MT=%s"
        euc-japan)
       ("excite-ja"
-       "http://www.excite.co.jp/search.gw?target=combined&look=excite_jp\
+       "https://www.excite.co.jp/search.gw?target=combined&look=excite_jp\
 &lang=jp&tsug=-1&csug=-1&search=%s"
        shift_jis)
       ("altavista"
-       "http://altavista.com/sites/search/web?q=\"%s\"&kl=ja&search=Search")
+       "https://altavista.com/sites/search/web?q=\"%s\"&kl=ja&search=Search")
       ("rpmfind"
-       "http://rpmfind.net/linux/rpm2html/search.php?query=%s"
+       "https://rpmfind.net/linux/rpm2html/search.php?query=%s"
        nil)
       ("debian-pkg"
-       "http://packages.debian.org/cgi-bin/search_contents.pl\
-?directories=yes&arch=i386&version=unstable&case=insensitive&word=%s")
+       "https://packages.debian.org/search\
+?&searchon=names&suite=stable&section=all&arch=amd64&keywords=%s")
       ("debian-bts"
-       "http://bugs.debian.org/cgi-bin/pkgreport.cgi?archive=yes&pkg=%s")
+       "https://bugs.debian.org/cgi-bin/pkgreport.cgi?archive=yes&pkg=%s")
       ("freebsd-users-jp"
-       "http://home.jp.FreeBSD.org/cgi-bin/namazu.cgi?key=\"%s\"&whence=0\
+       "https://home.jp.FreeBSD.org/cgi-bin/namazu.cgi?key=\"%s\"&whence=0\
 &max=50&format=long&sort=score&dbname=FreeBSD-users-jp"
        euc-japan)
       ("iij-archie"
-       "http://www.iij.ad.jp/cgi-bin/archieplexform?query=%s\
+       "https://www.iij.ad.jp/cgi-bin/archieplexform?query=%s\
 &type=Case+Insensitive+Substring+Match&order=host&server=archie1.iij.ad.jp\
 &hits=95&nice=Nice")
       ("waei"
-       "http://dictionary.goo.ne.jp/search.php?MT=%s&kind=je"
+       "https://dictionary.goo.ne.jp/search.php?MT=%s&kind=je"
        euc-japan)
       ("eiwa"
-       "http://dictionary.goo.ne.jp/search.php?MT=%s&kind=ej")
+       "https://dictionary.goo.ne.jp/search.php?MT=%s&kind=ej")
       ("kokugo"
-       "http://dictionary.goo.ne.jp/search.php?MT=%s&kind=jn"
+       "https://dictionary.goo.ne.jp/search.php?MT=%s&kind=jn"
        euc-japan)
       ("eiei"
-       "http://www.dictionary.com/cgi-bin/dict.pl?term=%s&r=67")
+       "https://www.dictionary.com/cgi-bin/dict.pl?term=%s&r=67")
       ,@(if ja
 	    '(("amazon"
-	       "http://www.amazon.co.jp/gp/search?\
+	       "https://www.amazon.co.jp/gp/search?\
 __mk_ja_JP=%%83J%%83%%5E%%83J%%83i&url=search-alias%%3Daps&field-keywords=%s"
 	       shift_jis)
 	      ("amazon-en"
-	       "http://www.amazon.com/exec/obidos/search-handle-form/\
+	       "https://www.amazon.com/exec/obidos/search-handle-form/\
 250-7496892-7797857"
 	       iso-8859-1
 	       "url=index=blended&field-keywords=%s"))
 	  '(("amazon"
-	     "http://www.amazon.com/exec/obidos/search-handle-form/\
+	     "https://www.amazon.com/exec/obidos/search-handle-form/\
 250-7496892-7797857"
 	     iso-8859-1
 	     "url=index=blended&field-keywords=%s")
 	    ("amazon-ja"
-	       "http://www.amazon.co.jp/gp/search?\
+	     "https://www.amazon.co.jp/gp/search?\
 __mk_ja_JP=%%83J%%83%%5E%%83J%%83i&url=search-alias%%3Daps&field-keywords=%s"
-	       shift_jis)))
-      ("emacswiki" "http://www.emacswiki.org/cgi-bin/wiki?search=%s")
-      ("en.wikipedia" "http://en.wikipedia.org/wiki/Special:Search?search=%s")
-      ("de.wikipedia" "http://de.wikipedia.org/wiki/Spezial:Search?search=%s"
+	     shift_jis)))
+      ("emacswiki" "https://www.emacswiki.org/cgi-bin/wiki?search=%s")
+      ("en.wikipedia" "https://en.wikipedia.org/wiki/Special:Search?search=%s")
+      ("de.wikipedia" "https://de.wikipedia.org/wiki/Spezial:Search?search=%s"
        utf-8)
-      ("ja.wikipedia" "http://ja.wikipedia.org/wiki/Special:Search?search=%s"
+      ("ja.wikipedia" "https://ja.wikipedia.org/wiki/Special:Search?search=%s"
        utf-8)
-      ("msdn" "http://search.msdn.microsoft.com/search/default.aspx?query=%s")
-      ("freshmeat" "http://freshmeat.net/search/?q=%s&section=projects")))
+      ("msdn" "https://search.msdn.microsoft.com/search/default.aspx?query=%s")
+      ("duckduckgo" "https://duckduckgo.com/?q=%s" utf-8)))
   "*An alist of search engines.
 Each element looks like (ENGINE ACTION CODING POST-DATA)
 ENGINE is a string, the name of the search engine.
@@ -239,20 +226,23 @@ If CODING is omitted, it defaults to `w3m-default-coding-system'."
   :group 'w3m
   :type `(repeat
 	  (group :indent 2
-		 (string :format "Engine: %v\n" :size 0)
-		 (string :format "       Action: %v\n" :size 0)
-		 (coding-system :format "%t: %v\n" :size 0)
+		 (string :format "Engine: %v\n")
+		 (string :format "       Action: %v\n")
+		 (coding-system :format "%t: %v\n")
 		 (checklist :inline t
 			    :entry-format ,(if (w3m-device-on-window-system-p)
 					       "%b   %v"
 					     "%b  %v")
-			    (string :format "PostData: %v\n" :size 0)))))
+			    (string :format "PostData: %v\n")))))
 
 (defcustom w3m-search-default-engine "google"
-  "*Name of the default search engine.
+  "Name of the default search engine.
 See also `w3m-search-engine-alist'."
   :group 'w3m
-  :type '(string :size 0))
+  :type (nconc '(radio) (mapcar
+			 (lambda (x)
+			   (list 'const (w3m-substring-no-properties (car x))))
+			 w3m-search-engine-alist)))
 
 (defcustom w3m-search-word-at-point t
   "*Non-nil means that the word at point is used as an initial string.
@@ -297,26 +287,27 @@ PROMPT-WITH-DEFAULT instead of string PROMPT."
 		   prompt)
 		 initial history default)))
 
-(defun w3m-search-read-variables ()
-  "Ask for a search engine and words to query and return them as a list."
-  (when w3m-current-process
-    (error "%s"
-	   (substitute-command-keys "
-Cannot run two w3m processes simultaneously \
-\(Type `\\<w3m-mode-map>\\[w3m-process-stop]' to stop asynchronous process)")))
-  (let* ((search-engine
+(defun w3m-search-read-variables (where)
+  "Ask for a search engine and words to query and return them as a list.
+
+WHERE is a string which should have the value \"current\" is the calling
+function intends for the search results to be presented in the current
+buffer, or \"new\" if in a new buffer."
+  (w3m--buffer-busy-error)
+  (let* ((prompt-prefix (format "Search in %s buffer. " where))
+	 (search-engine
 	  (if current-prefix-arg
 	      (let ((default (or (car w3m-search-engine-history)
 				 w3m-search-default-engine))
 		    (completion-ignore-case t))
-		(completing-read (format "Which engine? (default %s): "
-					 default)
+		(completing-read (format "%sWhich engine? (default %s): "
+					 prompt-prefix default)
 				 w3m-search-engine-alist nil t nil
 				 'w3m-search-engine-history default))
 	    w3m-search-default-engine))
 	 (query
 	  (w3m-search-read-query
-	   (format "%s search: " search-engine)
+	   (format "%s %s search: " prompt-prefix search-engine)
 	   (format "%s search (default %%s): " search-engine))))
     (list search-engine query)))
 
@@ -339,18 +330,21 @@ Cannot run two w3m processes simultaneously \
 ;;;###autoload
 (defun w3m-search (search-engine query)
   "Search QUERY using SEARCH-ENGINE.
+
+Search results will appear in the current buffer.
+
 When called interactively with a prefix argument, you can choose one of
 the search engines defined in `w3m-search-engine-alist'.  Otherwise use
 `w3m-search-default-engine'.
 If Transient Mark mode, use the region as an initial string of query
 and deactivate the mark."
-  (interactive (w3m-search-read-variables))
+  (interactive (w3m-search-read-variables "current"))
   (w3m-search-do-search 'w3m-goto-url search-engine query))
 
 ;;;###autoload
 (defun w3m-search-new-session (search-engine query)
-  "Like `w3m-search', but do the search in a new session."
-  (interactive (w3m-search-read-variables))
+  "Like `w3m-search', but do the search in a new buffer."
+  (interactive (w3m-search-read-variables "new"))
   (w3m-search-do-search 'w3m-goto-url-new-session search-engine query))
 
 ;;;###autoload
