@@ -1,9 +1,11 @@
 ;;; w3m-session.el --- Functions to operate session of w3m -*- coding: iso-2022-7bit; -*-
 
-;; Copyright (C) 2001-2003, 2005-2013 TSUCHIYA Masatoshi <tsuchiya@namazu.org>
+;; Copyright (C) 2001-2003, 2005-2013, 2017, 2018
+;; TSUCHIYA Masatoshi <tsuchiya@namazu.org>
 
 ;; Author: Hideyuki SHIRAI <shirai@meadowy.org>
 ;; Keywords: w3m, WWW, hypermedia
+;; Homepage: http://emacs-w3m.namazu.org/
 
 ;; This program is free software; you can redistribute it and/or modify
 ;; it under the terms of the GNU General Public License as published by
@@ -22,10 +24,64 @@
 
 ;;; Commentary:
 
-;; w3m-session.el is the add-on program of emacs-w3m to save and load
-;; sessions.   For more detail about emacs-w3m, see:
-;;
-;;    http://emacs-w3m.namazu.org/
+;; w3m-session.el is an add-on program of emacs-w3m to save, load,
+;; delete, and rename sessions or their selected tabs.
+
+;; You need not load or restore an entire session.  You can examine and
+;; manage the contents (individual tabs) of any session, including
+;; deleting, renaming, or loading them one at a time.
+
+;; A session is any collection of buffers (tabs) open at any given
+;; time.  Be aware, however, that unfortunately much of the code and
+;; documentation uses the terms session and buffer interchangeably.
+;; There is also a term 'session-group' which is identical to a
+;; session which has more than buffer.  Yes, this does need to be
+;; refactored!
+
+;; Session management does its best to automatically keep track of the
+;; current collection of w3m buffers so that should a catastrophic
+;; crash occur, the entire session can be restored.  Upon restarting
+;; emacs-w3m after a crash, the user will be prompted whether to
+;; recover the prior session.  Additionally, the user may at any time
+;; manually save the current session, or restore the tabs of a prior
+;; session.  Note that restoring a prior session is not destructive; it
+;; only adds new tabs into the current session for each tab of the
+;; saved session.
+
+;; Session information is also saved when one quits emacs-w3m using
+;; the `w3m-quit' command, bound by default to `Q'.
+
+;; The session information for all sessions are stored as a single
+;; elisp object, by default in file ~/.w3m/.session.  Tab content is
+;; NOT stored.  What is stored for each tab is the url, title, cursor
+;; position, and tab history.  Even so, this can eventually become a
+;; monstrously large data structure, and probably needlessly, if one
+;; accumulates large history records for each tab.
+
+;;;; Typical usage and tutorial
+
+;; One enters session management by evaluating `w3m-session-select'
+;; (M-s).  Typically, there will be at least one session automatically
+;; present, either labeled as a crash recovery session or an
+;; automatically saved one.  Next to each session name will be a number
+;; in brackets representing the number of tab (buffers) in the
+;; session.  There will also be column entries for the session
+;; timestamp, and summary URL entries.  One exits session management
+;; with either `q' or `C-g'.
+
+;; From any emacs-w3m buffer one may save a session by evaluating
+;; `w3m-session-save' (M-S).
+
+;; From within session management, one can rename (`r') or delete
+;; (`d') any session.  Selecting a session using the RET key will load
+;; ALL of that session's tabs.  You can examine the details of a
+;; session by pressing `M-s' (the same key binding you used to enter
+;; the session management).  At this point, you can rename, delete, or
+;; load any individual tab.  Note that the first entry on the list
+;; gives you a second chance to load ALL the tabs.  Quitting returns
+;; you to the session list.
+
+;; All changes are saved to disk immediately.
 
 
 ;;; Code:
@@ -47,11 +103,23 @@
   (autoload 'w3m-load-list "w3m")
   (autoload 'w3m-save-list "w3m"))
 
+(defvar w3m-session-group-open nil
+  "Which session-group is open.
+
+If a session-group is currently open, i.e., when displaying a
+list of buffers for an individual session, this should be set
+to the session (session-group) number.
+
+There is a legacy terminology problem that needs to be addressed
+here.  The documentation and symbol names currently confuse
+'sessions', 'buffers`, and 'session-groups'.  A 'session-group'
+is identical to a 'session' that has more than one 'buffer'.")
+
 (defcustom w3m-session-file
   (expand-file-name ".sessions" w3m-profile-directory)
   "*File name to keep sessions."
   :group 'w3m
-  :type '(file :size 0))
+  :type 'file)
 
 (defcustom w3m-session-autosave t
   "*Non-nil means save automatically when w3m quit."
@@ -64,7 +132,7 @@
   :type 'boolean)
 
 (defcustom w3m-session-crash-recovery t
-  "*Non-nil means emacs-w3m save session set automatically, and recover it when emacs-w3m crash."
+  "If non-nil, recover session by auto-saved one when emacs-w3m crashes."
   :group 'w3m
   :type 'boolean)
 
@@ -75,7 +143,7 @@
     "%Y-%m-%d (%a) %H:%M")
   "*Format of saved time."
   :group 'w3m
-  :type '(string :size 0))
+  :type 'string)
 
 (defcustom w3m-session-automatic-title
   (if (equal "Japanese" w3m-language)
@@ -83,7 +151,7 @@
     "Automatic saved sessions")
   "*String of title to save session automatically."
   :group 'w3m
-  :type '(string :size 0))
+  :type 'string)
 
 (defcustom w3m-session-deleted-title
   (if (equal "Japanese" w3m-language)
@@ -91,7 +159,7 @@
     "Removed sessions")
   "*String of title to save session when buffer delete."
   :group 'w3m
-  :type '(string :size 0))
+  :type 'string)
 
 (defcustom w3m-session-crash-recovery-title
   (if (equal "Japanese" w3m-language)
@@ -99,41 +167,47 @@
     "Crash recovery sessions")
   "*String of title to save session to use for crash recovering."
   :group 'w3m
-  :type '(string :size 0))
+  :type 'string)
 
 (defcustom w3m-session-deleted-keep-number 5
   "*Number to keep sessions when buffers delete."
   :group 'w3m
-  :type '(integer :size 0))
+  :type 'integer)
 
 (defcustom w3m-session-automatic-keep-number 5
   "*Number to keep sessions automatically."
   :group 'w3m
-  :type '(integer :size 0))
+  :type 'integer)
 
 (defcustom w3m-session-unknown-title "<Unknown Title>"
   "*String of title to use when title is not specified."
   :group 'w3m
-  :type '(string :size 0))
+  :type 'string)
 
 (defcustom w3m-session-load-last-sessions nil
-  "*Whether to load the last sessions when emacs-w3m starts."
+  "*Whether to re-load the most recent session when emacs-w3m
+starts."
   :group 'w3m
-  :type '(radio (const :format "Load the last sessions automatically." t)
-		(const :format "Ask whether to load the last sessions." ask)
-		(const :format "Never load the last sessions automatically." nil)))
+  :type
+  '(radio
+    (const :format "Re-load the last session automatically." t)
+    (const :format "Ask whether to re-load the last session." ask)
+    (const :format "Never re-load the last session automatically." nil)))
 
 (defcustom w3m-session-load-crashed-sessions 'ask
-  "*Whether to load the crashed sessions when emacs-w3m starts."
+  "*Whether to re-load a crashed session when emacs-w3m starts.
+This is used when emacs-w3m determines that the most recent session crashed."
   :group 'w3m
-  :type '(radio (const :format "Load the crashed sessions automatically." t)
-		(const :format "Ask whether to load the crashed sessions." ask)
-		(const :format "Never load the crashed sessions automatically." nil)))
+  :type
+  '(radio
+    (const :format "RelLoad the crashed session automatically." t)
+    (const :format "Ask whether to re-load the crashed session." ask)
+    (const :format "Never re-load the crashed session automatically." nil)))
 
 (defface w3m-session-select
-  `((((class color) (background light) (type tty))
+  `((((class color) (background light) (type nil))
      (:foreground "black"))
-    (((class color) (background dark) (type tty))
+    (((class color) (background dark) (type nil))
      (:foreground "cyan"))
     (((class color) (background light))
      (:foreground "dark blue"))
@@ -146,9 +220,9 @@
 (put 'w3m-session-select-face 'face-alias 'w3m-session-select)
 
 (defface w3m-session-selected
-  `((((class color) (background light) (type tty))
+  `((((class color) (background light) (type nil))
      (:foreground "blue" :bold t :underline t))
-    (((class color) (background dark) (type tty))
+    (((class color) (background dark) (type nil))
      (:foreground "cyan" :bold t :underline t))
     (((class color) (background light))
      (:foreground "dark blue" :bold t :underline t))
@@ -170,7 +244,7 @@
 		 (list x)))
 	     (copy-sequence (w3m-history-slimmed-history-flat))))))
 
-;; format of sessin file.
+;; format of session file.
 ;; '((sessiontitle1 time1 ((url11 pos11 hflat11 urltitle11)
 ;;                         (url12 pos12 hflat12 urltitle12) ...) current1)
 ;;   ...
@@ -182,22 +256,26 @@
      (error
       (if (and (file-exists-p w3m-session-file)
 	       (yes-or-no-p (format
-			     "\
-Sorry, an error found in \"%s\"; may we remove it? "
+			     "An error was found in \"%s\"; may we remove it? "
 			     ,(if (featurep 'xemacs)
 				  '(abbreviate-file-name w3m-session-file t)
 				'(abbreviate-file-name w3m-session-file)))))
 	  (progn
 	    (delete-file w3m-session-file)
 	    (run-at-time 0.1 nil #'message
-			 "\"%s\" has been removed; try again"
+			 "\"%s\" has been removed; try again."
 			 (abbreviate-file-name w3m-session-file))
 	    (keyboard-quit))
 	(signal (car err) (cdr err))))))
 
 ;;;###autoload
 (defun w3m-session-save ()
-  "Save list of displayed session."
+  "Save the current session (all currently open emacs-w3m buffers).
+
+The user will be prompted for a name for the saved session.  The
+saved session information will include, for each currently open
+emacs-w3m buffer: the current url and page title, and the
+buffer's url history."
   (interactive)
   (w3m-session-ignore-errors
    (let ((sessions (w3m-load-list w3m-session-file))
@@ -215,12 +293,14 @@ Sorry, an error found in \"%s\"; may we remove it? "
      (setq titles (cons (cons title title) titles))
      (catch 'loop
        (while t
-	 (setq title (completing-read prompt titles nil nil title))
+	 ;; A devious way to emulate INITIAL-INPUT that is deprecated.
+	 (let ((minibuffer-setup-hook (lambda nil (insert title))))
+	   (setq title (completing-read prompt titles nil nil nil nil title)))
 	 (if (or (string= title "")
 		 (and (assoc title sessions)
-		      (not (y-or-n-p (format "\"%s\" is exist. Overwrite? "
+		      (not (y-or-n-p (format "\"%s\" exists.  Overwrite? "
 					     title)))))
-	     (setq prompt "Again New session title: ")
+	     (setq prompt "New session title: ")
 	   (throw 'loop t))))
      (setq cbuf (current-buffer))
      (save-current-buffer
@@ -237,16 +317,17 @@ Sorry, an error found in \"%s\"; may we remove it? "
 				  w3m-current-title)
 			    urls)))))
      (if (not urls)
-	 (message "%s: no session save...done" title)
+	 (message "%s: no buffers saved...done" title)
        (setq len (length urls))
        (setq urls (nreverse urls))
        (when (assoc title sessions)
-	 (setq sessions (delete (assoc title sessions) sessions)))
+	 (setq sessions (delq (assoc title sessions) sessions)))
        (setq sessions (cons (list title (current-time) urls cnum) sessions))
        (w3m-save-list w3m-session-file sessions)
-       (if (= len 1)
-	   (message "%s: 1 session save...done" title)
-	 (message "%s: %d sessions save...done" title len))))))
+       (message "%s: %d buffer%s saved...done" title len (if (= len 1) "" "s"))
+       (when (and (setq buf (get-buffer " *w3m-session select*"))
+		  (get-buffer-window buf 'visible))
+	 (save-selected-window (w3m-session-select)))))))
 
 (defun w3m-session-automatic-save ()
   "Save list of displayed session automatically."
@@ -360,7 +441,7 @@ Sorry, an error found in \"%s\"; may we remove it? "
 	 (when urls
 	   (setq urls (nreverse urls))
 	   (setq tmp (assoc title sessions))
-	   (when tmp (setq sessions (delete tmp sessions)))
+	   (when tmp (setq sessions (delq tmp sessions)))
 	   (setq sessions (cons (list title (current-time) urls nil) sessions))
 	   (w3m-save-list w3m-session-file sessions)))))))
 
@@ -372,7 +453,7 @@ Sorry, an error found in \"%s\"; may we remove it? "
      (let* ((sessions (w3m-load-list w3m-session-file))
 	    (item (assoc w3m-session-crash-recovery-title sessions)))
        (when item
-	 (setq sessions (delete item sessions))
+	 (setq sessions (delq item sessions))
 	 (w3m-save-list w3m-session-file sessions))))))
 
 (defvar w3m-session-select-mode-map nil)
@@ -401,9 +482,7 @@ Sorry, an error found in \"%s\"; may we remove it? "
     (setq w3m-session-select-mode-map map)))
 
 ;;; Local variables
-(defvar w3m-session-select-wincfg nil)
 (defvar w3m-session-select-sessions nil)
-(make-variable-buffer-local 'w3m-session-select-wincfg)
 (make-variable-buffer-local 'w3m-session-select-sessions)
 
 (defun w3m-session-select-mode (&optional sessions)
@@ -429,16 +508,30 @@ Sorry, an error found in \"%s\"; may we remove it? "
 	   major-mode 'w3m-session-select-mode
 	   w3m-session-select-sessions sessions
 	   buffer-read-only t)
+     (setq w3m-session-group-open nil)
      (use-local-map w3m-session-select-mode-map)
-     (w3m-session-select-list-all-sessions))))
+     (w3m-session-select-list-all-sessions)
+     (add-hook 'pre-command-hook 'w3m--session-update-faces t t)
+     (add-hook 'post-command-hook 'w3m--session-update-faces t t))))
+
+(defun w3m--session-update-faces ()
+  "A hook function for `w3m-session-select' buffers.
+Meant for use  with  `pre-command-hook' and `post-command-hook'."
+  (let ((beg (line-beginning-position))
+	(inhibit-read-only t))
+   (put-text-property beg (next-single-property-change beg 'w3m-session-number)
+     'face
+     (if (equal (get-text-property beg 'face) 'w3m-session-select)
+       'w3m-session-selected
+      'w3m-session-select))))
 
 (defun w3m-session-select-list-all-sessions ()
-  "List up all saved sessions."
+  "List all saved sessions."
   (let ((sessions w3m-session-select-sessions)
 	(num 0)
 	(max 0)
 	(inhibit-read-only t)
-	title titles time times url urls wid pos)
+	title titles time times wid pos)
     (if (not sessions)
 	(progn
 	  (message "No saved session")
@@ -451,26 +544,16 @@ Sorry, an error found in \"%s\"; may we remove it? "
 	      (setq titles (cons title titles))
 	      (setq times (cons (format-time-string w3m-session-time-format
 						    (nth 1 x))
-				times))
-	      (setq urls (cons (mapconcat (lambda (url)
-					    (if (stringp url)
-						url
-					      (car url)))
-					  (nth 2 x) ", ")
-			       urls)))
+				times)))
 	    sessions)
       (setq titles (nreverse titles))
       (setq times (nreverse times))
-      (setq urls (nreverse urls))
       (setq max (+ max 2))
       (erase-buffer)
-      (insert "Select session:\n\n")
       (while (and (setq title (car titles))
-		  (setq time (car times))
-		  (setq url (car urls)))
+		  (setq time (car times)))
 	(setq titles (cdr titles))
 	(setq times (cdr times))
-	(setq urls (cdr urls))
 	(setq pos (point))
 	(insert title)
 	(add-text-properties pos (point)
@@ -478,18 +561,18 @@ Sorry, an error found in \"%s\"; may we remove it? "
 				    w3m-session-number ,num))
 	(setq num (1+ num))
 	(insert (make-string (- max (string-width title)) ?\ ))
-	(insert time "  " url "\n"))
+	(insert time "\n"))
+      (delete-char -1)
       (goto-char (point-min))
-      (goto-char (next-single-property-change
-		  (point) 'w3m-session-number))
-      (put-text-property (point)
-			 (next-single-property-change
-			  (point) 'w3m-session-number)
-			 'face 'w3m-session-selected)
       (set-buffer-modified-p nil)
       (setq buffer-read-only t))))
 
 (defun w3m-session-select-list-session-group (arg)
+  "List all buffers (i.e., tabs) within a session.
+
+The list can be acted upon similarly to a session list, i.e.,
+entries can be individually deleted, renamed, or opened as a new
+buffer in the current session."
   (let ((session (nth 2 (nth arg w3m-session-select-sessions)))
 	(num 0)
 	(max 0)
@@ -498,7 +581,8 @@ Sorry, an error found in \"%s\"; may we remove it? "
 	titles urls pos)
     (when session
       (mapc (lambda (x)
-	      (setq title (format "%s" (or (nth 3 x) w3m-session-unknown-title)))
+	      (setq title
+		    (format "%s" (or (nth 3 x) w3m-session-unknown-title)))
 	      (setq wid (string-width title))
 	      (when (> wid max)
 		(setq max wid))
@@ -539,44 +623,27 @@ Sorry, an error found in \"%s\"; may we remove it? "
   "Move the point to the next session."
   (interactive "p")
   (unless arg (setq arg 1))
-  (let ((positive (< 0 arg))
-	(inhibit-read-only t))
-    (beginning-of-line)
-    (put-text-property (point)
-		       (next-single-property-change
-			(point) 'w3m-session-number)
-		       'face 'w3m-session-select)
-    (while (not (zerop arg))
-      (forward-line (if positive 1 -1))
-      (unless (get-text-property (point) 'w3m-session-number)
-	(if positive
-	    (goto-char (next-single-property-change
-			(point-min) 'w3m-session-number))
-	  (goto-char (previous-single-property-change
-		      (point-max) 'w3m-session-number))))
-      (setq arg (if positive
-		    (1- arg)
-		  (1+ arg))))
-    (beginning-of-line)
-    (put-text-property (point)
-		       (next-single-property-change
-			(point) 'w3m-session-number)
-		       'face 'w3m-session-selected)
+  (let ((target (1+ (mod (1- (+ arg (line-number-at-pos (point))))
+			 (line-number-at-pos (point-max))))))
+    (goto-char (point-min))
+    (forward-line (1- target))
     (set-buffer-modified-p nil)))
 
 (defun w3m-session-select-previous (&optional arg)
-  "the point to the previous session."
+  "Move the point to the previous session."
   (interactive "p")
   (w3m-session-select-next (- arg)))
 
 (defun w3m-session-select-quit ()
   "Exit from w3m session select mode."
   (interactive)
-  (let ((buffer (current-buffer))
-	(wincfg w3m-session-select-wincfg))
-    (or (one-window-p) (delete-window))
-    (kill-buffer buffer)
-    (set-window-configuration wincfg)))
+  (if w3m-session-group-open
+      (let ((num w3m-session-group-open))
+	(setq w3m-session-group-open nil)
+	(w3m-session-select-list-all-sessions))
+    (let ((buffer (current-buffer)))
+      (or (one-window-p) (delete-window))
+      (kill-buffer buffer))))
 
 (defun w3m-session-select-select ()
   "Select the session."
@@ -584,30 +651,30 @@ Sorry, an error found in \"%s\"; may we remove it? "
   (beginning-of-line)
   (let* ((num (get-text-property
 	       (point) 'w3m-session-number))
-	 (item (if (consp num) 
+	 (item (if (consp num)
 		   (nth (cdr num)
-			(caddr (nth (car num) 
+			(caddr (nth (car num)
 				    w3m-session-select-sessions)))
 		 (nth num w3m-session-select-sessions)))
 	 (session (if (consp num)
 		      (list (or (cadddr item) w3m-session-unknown-title)
 			    nil
-			    (list item) 
+			    (list item)
 			    nil)
 		    item)))
     (w3m-session-select-quit)
     (w3m-session-goto-session session)))
 
-(defun w3m-session-select-open-session-group ()
+(defun w3m-session-select-open-session-group (&optional arg)
   "Open the session group."
   (interactive)
   (beginning-of-line)
-  (let ((num (get-text-property
-	      (point) 'w3m-session-number))
+  (let ((num (or arg (get-text-property (point) 'w3m-session-number)))
 	wheight)
     (if (consp num)
-	(message "There is no session group.")
-      (setq wheight 
+	(message "This is not a session group.")
+      (setq w3m-session-group-open num)
+      (setq wheight
 	    (max (+ (length (caddr (nth num w3m-session-select-sessions))) 6)
 		 window-min-height))
       (condition-case nil
@@ -618,7 +685,7 @@ Sorry, an error found in \"%s\"; may we remove it? "
 (defun w3m-session-select-save ()
   "Save the session."
   (interactive)
-  (when (y-or-n-p "Save this sessions? ")
+  (when (y-or-n-p "Save this session? ")
     (w3m-session-select-quit)
     (w3m-session-save)
     (w3m-session-select)))
@@ -630,54 +697,52 @@ Sorry, an error found in \"%s\"; may we remove it? "
   (let ((num (get-text-property
 	      (point) 'w3m-session-number))
 	(sessions w3m-session-select-sessions))
-    (w3m-session-select-quit)
     (w3m-session-rename sessions num)
-    (w3m-session-select)))
+    (if (not w3m-session-group-open)
+	(w3m-session-select num)
+      (w3m-session-select-open-session-group w3m-session-group-open))))
 
 (defun w3m-session-select-delete ()
-  "Delete the session."
+  "Delete an entry (either a session or a buffer)."
   (interactive)
-  (when (y-or-n-p "Delete this session? ")
+  (when (y-or-n-p "Delete this entry? ")
     (beginning-of-line)
     (let ((num (get-text-property
 		(point) 'w3m-session-number))
 	  (sessions w3m-session-select-sessions))
-      (w3m-session-select-quit)
       (w3m-session-delete sessions num)
-      (w3m-session-select))))
+      ;;(w3m-session-select)
+      ;;(forward-line (min num (- (line-number-at-pos (point-max)) 4))))))
+      (if (not w3m-session-group-open)
+	  (w3m-session-select (min num (1- (length sessions))))
+	(w3m-session-select-open-session-group w3m-session-group-open)
+	(forward-line (min (cdr num)
+			   (- (line-number-at-pos (point-max)) 4)))))))
 
 ;;;###autoload
-(defun w3m-session-select ()
-  "Select session from session list."
-  (interactive)
+(defun w3m-session-select (&optional n toggle nomsg)
+  "Select session from session list.
+Position point at N-th session if N is given.  With the
+prefix-argument, toggles the position of the popup window between
+being below or beside the main window."
+  (interactive (list nil current-prefix-arg nil))
   (w3m-session-ignore-errors
-   (let* ((sessions (w3m-load-list w3m-session-file))
-	  (showbuf (w3m-get-buffer-create " *w3m-session select*"))
-	  (wheight (max (+ (length sessions) 5) window-min-height))
-	  (wincfg (current-window-configuration))
-	  window last-window)
-     (setq last-window (previous-window
-			(w3m-static-if (fboundp 'frame-highest-window)
-			    (frame-highest-window)
-			  (frame-first-window))))
-     (while (minibuffer-window-active-p last-window)
-       (setq last-window (previous-window last-window)))
-     (while (and
-	     (not (one-window-p))
-	     (or (< (window-width last-window)
-		    (frame-width))
-		 (< (window-height last-window)
-		    (+ wheight window-min-height))))
-       (setq window last-window)
-       (setq last-window (previous-window window))
-       (delete-window window))
-     (select-window (split-window last-window))
-     (condition-case nil
-	 (shrink-window (- (window-height) wheight))
-       (error nil))
-     (switch-to-buffer showbuf)
-     (setq w3m-session-select-wincfg wincfg)
-     (w3m-session-select-mode sessions))))
+   (let ((sessions (w3m-load-list w3m-session-file))
+	 (showbuf " *w3m-session select*")
+	 window)
+     (if sessions
+	 (progn
+	   (w3m--setup-popup-window toggle showbuf nomsg)
+	   (w3m-session-select-mode sessions)
+	   (when n (w3m-session-select-next n)))
+       ;; else, ie. (not sessions)
+       (when (setq showbuf (get-buffer showbuf))
+	 (when (setq window (prog1 (get-buffer-window showbuf t)
+			      (kill-buffer showbuf)))
+	   (save-selected-window
+	     (select-window window)
+	     (or (one-window-p t t) (delete-window window)))))
+       (message "No saved session")))))
 
 (defun w3m-session-goto-session (session)
   "Goto URLs."
@@ -709,56 +774,79 @@ Sorry, an error found in \"%s\"; may we remove it? "
     (message "Session goto(%s)...done" title)))
 
 (defun w3m-session-rename (sessions num)
-    (if (consp num)
-	(message "This command can execute in Main session area")
-      (let ((prompt "New session title: ")
-	    (overwrite nil)
-	    tmp title otitle)
-	(setq tmp (nth num sessions))
-	(setq otitle (car tmp))
-	(setq title otitle)
-	(catch 'loop
-	  (while t
-	    (setq title (read-from-minibuffer prompt otitle))
-	    (cond
-	     ((string= title "")
-	      nil)
-	     ((string= title otitle)
-	      (when (y-or-n-p
-		     (format "\"%s\" is same as original title. Do not rename? "
-			     title))
-		(throw 'loop t)))
-	     ((assoc title sessions)
-	      (when (y-or-n-p (format "\"%s\" is exist. Overwrite? " title))
-		(setq overwrite t)
-		(throw 'loop t)))
-	     (t
-	      (throw 'loop t)))
-	    (setq prompt "Again New session title: ")))
-	(when overwrite
-	  (setq sessions (delete (assoc title sessions) sessions)))
-	(unless (string= title otitle)
-	  (setq sessions (delete tmp sessions))
-	  (setcar tmp title)
-	  (setq sessions (cons tmp sessions))
-	  (w3m-save-list w3m-session-file sessions)))))
+  "Rename an entry (either a session or a buffer).
+
+Rename session number NUM, when NUM is an integer.  NUM may also
+be a cons cell, for which the car is a session number and the cdr
+is a buffer entry (i.e., a tab) within that session.  In that case
+rename the buffer entry."
+  (let* ((default-prompt "Enter new session title (C-g to abort): ")
+	 (prompt default-prompt)
+	 overwrite
+	 title
+	 (group (if (consp num) (nth 2 (nth (car num) sessions)) nil))
+	 (tmp  (if group (nth (cdr num) group) (nth num sessions)))
+	 (otitle (if (consp num) (nth 2 (cdr tmp)) (car tmp))))
+    (while (not title)
+      ;; A devious way to emulate INITIAL-INPUT that is deprecated.
+      (let ((minibuffer-setup-hook (lambda nil (insert otitle))))
+	(setq title (read-from-minibuffer prompt nil nil nil nil otitle)))
+      (cond
+       ((string= title "")
+	(setq title nil
+	      prompt default-prompt))
+       ((string= title otitle)
+	(setq prompt (concat title
+			     " is same as original title (C-g to abort): ")
+	      title nil))
+       ((and (not group) (assoc title sessions))
+	(if (not (y-or-n-p (format "\"%s\" exists.  Overwrite? " title)))
+	    (setq prompt default-prompt
+		  title nil)
+	  (setq sessions (delq (assoc title sessions) sessions))
+	  (setq num (seq-position sessions (assoc otitle sessions)))))
+       ((and group (member title (mapcar (lambda (x) (car (last x))) group)))
+	;; handle *buffer* rename (overwrite existing)
+	;; within a session ("session-group")
+	(setq prompt "\
+Not yet supported.  Manually delete the other entry, or try again. "
+	      title nil))))
+    ;; in this case, wrapper must decrement its copy of num
+    ;; BB_2018-02-15: I don't understand that comment
+    (cond
+     (group ;; handle *buffer* rename within a session ("session-group")
+      (setf (nth 2 (cdr tmp)) title)
+      (setf (nth (cdr num) group) tmp)
+      (setf (nth 2 (nth (car num) sessions)) group))
+     ;; BB_2018-02-15: The good news is that this seems to be working;
+     ;; the bad news is that an examination of the .sessions file
+     ;; reveals a format difference, in that the original buffer name
+     ;; was encoded with text properties, like so:
+     ;;     #("w3m_home_page.html" 0 1 (idx 0))
+     ;; and the replacement is just a string.  The text property may
+     ;; just be cruft for the purpose of this file, but I'm not sure.
+     (t
+      (setcar tmp title)
+      (setcar (nthcdr num sessions) tmp)))
+    (w3m-save-list w3m-session-file sessions)))
 
 (defun w3m-session-delete (sessions num)
-  (let (tmp)
-    (if (consp num)
-	(let ((item (nth 2 (nth (car num) sessions))))
-	  (setq tmp (delete (nth (cdr num) item)
-			    item))
-	  (setf (nth 2 (nth (car num) sessions))
-		tmp))
-      (setq tmp (nth num sessions))
-      (setq sessions (delete tmp sessions)))
-    (if sessions
-	(w3m-save-list w3m-session-file sessions)
-      (let ((file (expand-file-name w3m-session-file)))
-	(when (and (file-exists-p file)
-		   (file-writable-p file))
-	  (delete-file file))))))
+  "Delete an entry (either a session or a buffer).
+
+Delete session number NUM, when NUM is an integer.  NUM may also
+be a cons cell, for which the car is a session number and the cdr
+is a buffer entry (i.e., a tab) within that session.  In that case
+delete the buffer entry."
+  (if (consp num)
+      (let* ((item (nth 2 (nth (car num) sessions)))
+	     (tmp (delq (nth (cdr num) item) item)))
+	(setf (nth 2 (nth (car num) sessions)) tmp))
+    (setq sessions (delq (nth num sessions) sessions)))
+  (if sessions
+      (w3m-save-list w3m-session-file sessions)
+    (let ((file (expand-file-name w3m-session-file)))
+      (when (and (file-exists-p file) (file-writable-p file))
+	(delete-file file)))))
 
 (defvar w3m-session-menu-items
   `([,(w3m-make-menu-item "$B?7$7$$%;%C%7%g%s$r:n$k(B..."
@@ -786,7 +874,7 @@ Sorry, an error found in \"%s\"; may we remove it? "
      w3m-session-save t]
     [,(w3m-make-menu-item "$B%;%C%7%g%s$rA*Br$9$k(B" "Select Sessions")
      w3m-session-select t])
-    "*List of the session menu items.")
+  "*List of the session menu items.")
 
 ;;;###autoload
 (defun w3m-setup-session-menu ()
@@ -816,8 +904,8 @@ Sorry, an error found in \"%s\"; may we remove it? "
       (easy-menu-define w3m-session-menu w3m-mode-map
 	"The menu kepmap for the emacs-w3m session."
 	(cons "Session" (if pages
-			     (append items '("----") pages)
-			   items)))
+			    (append items '("----") pages)
+			  items)))
       (w3m-static-when (featurep 'xemacs)
 	(when (setq items (car (find-menu-item current-menubar '("Session"))))
 	  (setcdr items (cdr w3m-session-menu))
@@ -906,6 +994,9 @@ file exists, otherwise nil."
 		      (eq w3m-session-load-crashed-sessions t)))
 	 item)))))
 
+(add-hook 'w3m-arrived-shutdown-functions 'w3m-session-crash-recovery-remove t)
+(add-hook 'w3m-arrived-shutdown-functions 'w3m-session-automatic-save t)
+
 (provide 'w3m-session)
 
 ;;; w3m-session.el ends here
